/****************************************/
/*  Class to resolve hostnames          */
/****************************************/
/* Start      : 10/5/98                 */
/* Program ID :                         */
/* Version    : 1.00                    */
/* End        :                         */
/* Last Update: 11/10/98                */
/* Comments   :                         */
/*                                      */
/****************************************/


#ifndef _VLIB_VRESOLV_H
#define _VLIB_VRESOLV_H

#include "netdb.h"
#include "sys/types.h"
#include "sys/socket.h"
#include "netinet/in.h"
#include "arpa/inet.h"
#include "string.h"

#ifdef _DEBUG_
#include "stdio.h"
#endif

#define _RESOLV_MAX_HOSTNAME 256
#define _RESOLV_MAX_HOSTNUM 16

typedef char RESOLV_HOSTNAME[_RESOLV_MAX_HOSTNAME];

class DNSHostName
{
 protected:
  int AddrType;   /* This is private for now */
 public:
  RESOLV_HOSTNAME Name[_RESOLV_MAX_HOSTNUM+1];
  RESOLV_HOSTNAME IP[_RESOLV_MAX_HOSTNUM+1];
  int NameCount;
  int IPCount;
 protected:
  void SetName(char *dst,const  char *src);
 public:
  DNSHostName();
  virtual ~DNSHostName();
  DNSHostName(DNSHostName &d);
  void Reset();
  void SetData(hostent *h);
#ifdef _DEBUG_
  void ShowData();
#endif
  DNSHostName &operator=(DNSHostName &d);
};

class ReverseResolve : public DNSHostName
{
 private:
 public:
 public:
  void Open();
  void Close();
  ReverseResolve();
  ~ReverseResolve();
  char *Resolve(const char *host);
  ReverseResolve &operator=(ReverseResolve &d);
};

class HostResolve : public DNSHostName
{
 private:
 public:
  struct in_addr addr;
 public:
  HostResolve();
  ~HostResolve();
  HostResolve(HostResolve &d);
  char *Resolve(const char *host);  // Return the IP in a string (use inet_addr)
  HostResolve operator=(HostResolve &d);
};

class HostName
{
 private:
 public:
  DNSHostName *hname;
  ReverseResolve rr;
  HostResolve hr;
 public:
  HostName();
  ~HostName();
  HostName(HostName &d);
  char *Resolve(const char *host); // Return the hostname if host is an IP,
                             // or the IP if host is a hostname
  char *GetIP(char *host);   // Return the IP of host (even if host is an IP)
  HostName operator=(HostName &d);
};

#endif
