/****************************************/
/*					*/
/****************************************/
/* Start      : 08/09/2002		*/
/* Version    : 1.00			*/
/* Last Update: 08/09/2002		*/
/****************************************/


#ifndef _VLIB_V_FILE_SELECTOR_H
#define _VLIB_V_FILE_SELECTOR_H

#define	SB_WAIT_READ	1
#define	SB_WAIT_WRITE	2
#define	SB_WAIT_CONNECT	4

#include "v/misc/nkeytable.h"

#include "stdlib.h"

typedef	int	(SELECTOR_HOOK)(int);

// For many FDs
class SELECTOR_BASE
{
 public:
			SELECTOR_BASE	() {}
	virtual		~SELECTOR_BASE	() {}
	virtual int	doRead		(int fd) { return(-1); }	// just unregister this
	virtual int	doWrite		(int fd) { return(-1); }
	virtual int	doExcept	(int fd) { return(-1); }
	virtual int	doConnect	(int fd) { return(-1); }	// same as doWrite
};

// For one FD
class SELECTOR_BASE_ONE
{
 public:
 	int	fd;
 	int	WaitRead;
	int	WaitWrite;
	int	WaitConnect;
 public:
			SELECTOR_BASE_ONE	() {WaitRead=WaitWrite=WaitConnect=0; fd=-1;}
	virtual		~SELECTOR_BASE_ONE	() {}
	virtual int	doRead		() { return(-1); }	// just unregister this
	virtual int	doWrite		() { return(-1); }
	virtual int	doExcept	() { return(-1); }
	virtual int	doConnect	() { return(-1); }	// same as doWrite
};


class SELECTOR
{
 public:
//////////////////////////////////////////////////////////////////////
/////////// CLNT /////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
	class CLNT
	{
	 private:
	 public:

	 	SELECTOR_BASE		*cls;		// the hook class or NULL
	 	SELECTOR_BASE_ONE	*cls_one;	// the hook class or NULL
		SELECTOR_HOOK		*onRead, *onWrite, *onExcept, *onConnect;
		int			fd;
		int			wait;		// used for cls (not for cls_one)
	 private:
		void	ddd(CLNT &d);
	 public:
			CLNT		();
			~CLNT		();
			CLNT		(CLNT &d);
		CLNT	operator=	(CLNT &d);

		int	CallRead	();
		int	CallWrite	();
		int	CallExcept	();
		
		int	NeedRead	();
		int	NeedWrite	();
	};
//////////////////////////////////////////////////////////////////////
 private:
	NKEYTABLE<CLNT>		clients;
 private:
	void		ddd		(SELECTOR &d);
 public:
//////////////////////////////////////////////////////////////////////

			SELECTOR	();
			~SELECTOR	();
			SELECTOR	(SELECTOR &d);
	SELECTOR	operator=	(SELECTOR &d);

//////////////////////////////////////////////////////////////////////

	int		add_fd		(int fd);	// Add an fd for selecting. Return 0: Ok, -1: Exists already
	int		del_fd		(int fd);	// Remove an fd. Return 0: Ok, -1: Not found

	int		set_class	(int fd, SELECTOR_BASE_ONE *cls_one);	// Select a class for hooks - 0: Ok, -1:FD not found

	int		set_class	(int fd, SELECTOR_BASE *cls);	// Select a class for hooks - 0: Ok, -1:FD not found
	int		set_class_read	(int fd, int n=1);	// Set a class to wait for reading (not for BASE_ONE) - 0: Ok, -1: FD not found
	int		set_class_write	(int fd, int n=1);	// Set a class to wait for writing (not for BASE_ONE) - 0: Ok, -1: FD not found
	int		set_class_connect(int fd, int n=1);	// Set a class to wait for connect() (not for BASE_ONE) - 0: Ok, -1: FD not found
	int		clear_class_rwc	(int fd);		// clear read/write/connect for an fd (class) (not for BASE_ONE)
	int		remove_class	(int fd);		// completely remove a class

	int		set_hook_read	(int fd, SELECTOR_HOOK *hook);	// Set a hook for reading - ret as above
	int		set_hook_write	(int fd, SELECTOR_HOOK *hook);	// Set a hook for writing - ret as above
	int		set_hook_except	(int fd, SELECTOR_HOOK *hook);	// Set a hook for exception - ret as above
	int		set_hook_connect(int fd, SELECTOR_HOOK *hook);	// Set a hook for connect() (Same as write) - ret as above
	int		clear_hooks	(int fd);			// Remove all hooks

	int		doit		(int timeout=0);	// timeout in milliseconds
	int		doit_select	(int timeout=0);	// timeout==-1 => no timeout
};

#endif

/*
	Registration with SELECTOR:

		Classes:
			They inherit SELECTOR_BASE overloading the do*()
			functions. They set flags using set_class_*().
			
		Classes_one:
			They inherit SELECTOR_BASE_ONE overloadint the do*()
			functions. They set variables Wait* and clear them
			when not needed.

		Functions:
			Functions are registered using set_hook_*().
			No flags are required. Registering a function
			for reading will cause it to wait for read.
			Hooks are cleared using set_hook_*(NULL)
			or clear_hooks()


	SELECTOR_BASE:
		Works for many fds
		Functions return:
			-1:	Error - stop select()ing/poll()ing on this fd

	SELECTOR_BASE_ONE:
		Same as SELECTOR_BASE. Works for one fd only

	Use:
		BASE_ONE:
			When you need a class to handle only one fd
		BASE:
			When you need a class to handle many fds
		hooks:
			When you don't want class-based hooks.
			hooks are: int myhook(int fd)
*/
