/****************************************/
/*  Session management for cgi's        */
/****************************************/
/* Start      : 15/12/99                */
/* Program ID :                         */
/* Version    : 1.00                    */
/* End        :                         */
/* Last Update: 15/12/99                */
/* Comments   :                         */
/*                                      */
/****************************************/

/*
	Class Session may be used, to have sessions in cgi programs.
	It uses class Info, to get the cookie that holds the session ID.
	If it is not found, it may be used to create another session and
	set the cookie. The default session ID cookie is named CLID
	and by default it will expire after 24 hours.

	To use it:

	Session myses;
	s_cgi *scgi;

	scgi=cgiInit();
	collect_info(scgi);
	ses.GetCookie();	// Tries to find cookie. Sets Session::found
	if (!ses.found)
	{
		ses.Generate();		// Generate another session cookie
		ses.SetCookie();	// Set the cookie
		ses.CreateSession();
	} else 
	{
		if (ses.Validate())
		{
			ses.GetSession();
		}
	}
	cgiHeader();

	when a session cookie is generated, it's session value is stored in Session::Value
	The Value can be up to 24 chars: IIIIIIIIDDDDDDDDRRRRRRRR
	where	I: The IP of the client in hex (if it cannot be determined, then it is 0000000000)
		D: The current date in seconds from 1/1/70, in hex
		R: A random number.

	Session::Validate checks the IP number stored in the session cookie with the current
	one of the client. Returns 1 if it matches or 0 if it doesn't. This should be used if
	session cookies should only be returned from the same host that were sent to.

	Session::CreateSession is a virtual function. Derived classes may overload it, to have their
	internal 'session creation' functions stored in the Session class. This is not calles
	automaticaly

	Session::GetSession() same as above.

	Session::found is set when Session::GetCookie() is called. It is 1 if the client send back
	a cookie (no matter if it is valid or not), 0 if it didn't and -1 if Session::GetCookie()
	is not called yet.

	The constructor Session::Session can have 2 arguments.
	1st: Path, witch is the path that the session cookie is set for. By default it is /
	2nd: Domain, witch is the domain for witch the session cookie is set for. By default it is NULL

	if any of those arguments is NULL, then the coresponding attribute is not send with the cookie
*/

#ifndef _VLIB_V_HTML_SESSION_H
#define _VLIB_V_HTML_SESSION_H

#ifndef SESSION_COOKIE
#	define SESSION_COOKIE	"CLID"
#endif

#ifndef MAX_AGE
#	define MAX_AGE	1L*60*60*24	// Max age of a cookie
#endif

#include "v/html/cgi.h"
#include "v/html/collect_info.h"
#include "v/misc/random.h"

class Session
{
 private:
 public:
 	char *Domain;
 	char *Path;
 	char Value[128];        // Session ID
 	int found;		// 1: if the session cookie was found, 0: if not, -1: GetCookie() was not called yet

 private:
 public:
 	Session(char *Path="/", char *Domain=NULL);
 	virtual ~Session();
 	void Generate();
 	int Validate(char *st=NULL);	// validate the Session ID, (using the current remote IP address)
 					// Return: 0: Error, 1: OK
 	void SetCookie(CGI *cgi);	// Set the cookie for session identification
 	void GetCookie();		// Get the cookie for session identification
 	virtual void CreateSession();	// Create a new session in the session database
 	virtual void GetSession();	// Retrieve the session from the database
};

#endif

