/****************************************/
/*  A spinlock using semaphores		*/
/****************************************/
/* Start      : 31/07/2001		*/
/* Program ID : 			*/
/* Version    : 1.01			*/
/* End        : 			*/
/* Last Update: 06/09/2001		*/
/* Comments   : 			*/
/*                                      */
/****************************************/

#include "lock.h"
#include "v/lconfig.h"

/*
	The starting key number, when unique is defined
*/
#define KEY_BASE	100
#define PERM		0700

#define MAX_RECREATE_TRIES	500

#ifndef HAVE_SEMUN
union semun
{
	int val;                    /* value for SETVAL */
	struct semid_ds *buf;       /* buffer for IPC_STAT, IPC_SET */
	unsigned short int *array;  /* array for GETALL, SETALL */
	struct seminfo *__buf;      /* buffer for IPC_INFO */
};
#endif


LOCK::LOCK(int tkey, int retry, int autocreate)
{
	lkey=tkey;
	semid=-1;
	this->autocreate=autocreate;

	create();
//	init();
	this->retry=retry;
}

LOCK::~LOCK()
{
//	destroy();
}

void LOCK::create(int n)
{
	int	k;

	if (semid!=-1)
		destroy();

	switch(lkey)
	{
		case LOCK_KEY_UNIQUE:
			k=KEY_BASE-1;
			do {
				k++;
				semid=semget(k,1,IPC_CREAT | IPC_EXCL | PERM);
			} while (semid<0);
			lkey=k;
			init(n);
			break;
		case LOCK_KEY_PRIVATE:
			lkey=IPC_PRIVATE;
			semid=semget(lkey,1,PERM);
//			semid=semget(k,1,IPC_CREAT | PERM);
			break;
		default:
			semid=semget(lkey,1,0);
			if (semid==-1)	// it doesn't exist
			{
				semid=semget(lkey,1,IPC_CREAT | PERM);
				init(n);
			}
			break;
	}
}

void LOCK::destroy()
{
	int	ret;
	semun	sun_;	// Sun Solaris defines sun (asdfalksdfh)

	if (semid!=-1)
	{
		ret=lock(0);
		if (ret!=LOCK_EAGAIN)
		{
			ret=semctl(semid,0,IPC_RMID,sun_);
		}
		semid=-1;
	}
}

void LOCK::init(int n)
{
	semun	sun_;
	int	ret;

	if ((semid!=-1) && (n>=0))
	{
		sun_.val=n;
		ret=semctl(semid,0,SETVAL,sun_);
	}
}

int LOCK::check()
{
	int	ret;
	semun	sun_;
	int	done;

	if (semid==-1) ret=0;
	else
	{
		do {
			done=1;
			ret=semctl(semid,0,GETVAL,sun_);
			if (ret==-1)	// error
			{
				ret=0;
				if (errno==EINTR)
					done=0;
			} else ret=1;
		} while (!done);
	}

	return(ret);
}

LOCK::operator int()
{
	return(check());
}

int LOCK::lock(int block)
{
	sembuf	sop[2];
	int	ret=LOCK_OK;
	int	done;

//	r=lock(0);

	if (semid!=-1)
	{
		do {
			do {
				done=1;
				sop[0].sem_num=0;
				sop[0].sem_op=0;
				sop[0].sem_flg=SEM_UNDO | IPC_NOWAIT;
//			sop[0].sem_flg=SEM_UNDO | (block ? 0 : IPC_NOWAIT);
				sop[1].sem_num=0;
				sop[1].sem_op=1;
				sop[1].sem_flg=SEM_UNDO | IPC_NOWAIT;
//			sop[1].sem_flg=SEM_UNDO | (block ? 0 : IPC_NOWAIT);
				ret=semop(semid,sop,2);
				if ((ret==-1) && (block) &&  (errno==EAGAIN))
				{
					sop[0].sem_num=0;
					sop[0].sem_op=0;
					sop[0].sem_flg=SEM_UNDO;
					ret=semop(semid,sop,1);
					done=0;
				}
			} while (!done);
			done=1;
			if (ret==-1)
			{
				switch(errno)
				{
					case EINTR:
						if (retry) done=0;
						else ret=LOCK_EINTR;
						break;
					case EAGAIN:
						ret=LOCK_EAGAIN;
						break;
					case EFAULT:
#ifdef HAVE_EIDRM
					case EIDRM:
#endif
					case EINVAL:
						semid=-1;
						if (autocreate)
						{
							create(-1);
							if (autocreate<MAX_RECREATE_TRIES)
							{
								autocreate++;
								ret=lock();
							} else ret=lock();
							autocreate=1;
							break;
						}
					default:
						ret=LOCK_UNAVAIL;
						break;
				}
	   		}
      		} while (!done);
	} else ret=LOCK_UNAVAIL;

	return(ret);
}

int LOCK::unlock()
{
	int	ret=LOCK_OK;
	int	done;
	sembuf	sop;

	if (semid!=-1)
	{
		do {
			done=1;
/*			sop.sem_num=0;
			sop.sem_op=1;
			sop.sem_flg=SEM_UNDO;*/
			sop.sem_num=0;
			sop.sem_op=-1;
			sop.sem_flg=SEM_UNDO | IPC_NOWAIT;	// IPC_NOWAIT Added 25/10/2001

			ret=semop(semid, &sop, 1);
			if (ret==-1)
			{
				switch(errno)
				{
					case EINTR:
						if (retry) done=0;
						else ret=LOCK_EINTR;
						break;
					default:
						ret=LOCK_UNAVAIL;
						break;
				}
			}
		} while (!done);
	} else ret=LOCK_UNAVAIL;

	return(ret);
}

int LOCK::key()
{
	return(lkey);
}
