/****************************************/
/*  A spinlock using semaphores		*/
/****************************************/
/* Start      : 31/07/2001		*/
/* Program ID : 			*/
/* Version    : 1.01			*/
/* End        : 			*/
/* Last Update: 06/09/2001		*/
/* Comments   : 			*/
/*                                      */
/****************************************/

#ifndef _VLIB_V_IPC_LOCK_H
#define _VLIB_V_IPC_LOCK_H

#include "sys/types.h"
#include "sys/ipc.h"
#include "sys/sem.h"
#include "errno.h"

#define LOCK_KEY_UNIQUE		-2
#define LOCK_KEY_PRIVATE	IPC_PRIVATE

#define LOCK_EAGAIN		-1
#define LOCK_UNAVAIL		-2 // the semaphore doesn't exist
#define LOCK_EINTR		-3 // the system call was interrupted
#define LOCK_OK			0

/*
	Implement a lokcing mechanism using semaphores

	tkey:	is the key to be used, or LOCK_KEY_PRIVATE, to be private, or LOCK_KEY_UNIQUE
		to auto-get an available key.

	retry:	1: when a system call is interrupted, it will be restarted (LOCK_EINTR will
		never be returned). 0: the funciton will return LOCK_EINTR

	autocreate:	1: When lock() is called and the semaphore doesn't exist, it will
			be created (With the existing key)

	when having more than 2 processes, with the same lock, LOCK_KEY_PRIVATE is no good.
	when a process die, the semaphore is removed and the other 2 (or more processes) will
	recreate a private semaphore, which will not be the same. 

	*NOTE* You must destroy the semaphore, using LOCK::destroy() when not needed (by hand)
*/
class LOCK
{
 private:
 	key_t	lkey;
	int	semid;
	int	retry;		// restart interrupted system calls
	int	autocreate;	// auto create (if needed) the semaphore, if it destroyed
				// ex from another process
 public:
 private:
	void	ddd		(LOCK &d);
	void	create		(int n=0);	// initial value
	void	init		(int n);	// initial value, -1 == don't set
	int	check		();	// return 1: if the lock is funcitonal, or 0: if it is not.. (semaphore unavailable)
 public:
		LOCK	(int tkey=LOCK_KEY_UNIQUE, int retry=1, int autocreate=1);
		~LOCK	();
		operator int	();		// return 0 if the lock is not created, or 1 if it is ok

	int	lock		(int block=1);	// if block!=0 , it will block untill locked
	int	unlock		();
	int	key		();	// return the key
					// NOTE: This DOES NOT mean the the semaphore is available.
					// When the semaphore is destroyed, the key will not change..
					// use operator int()
	void	destroy		();
};

#endif

