/****************************************/
/*  Message Queues			*/
/****************************************/
/* Start      : 22/08/2001		*/
/* Program ID : 			*/
/* Version    : 1.00			*/
/* End        : 			*/
/* Last Update: 22/08/2001		*/
/* Comments   : 			*/
/*                                      */
/****************************************/

#include "v/lconfig.h"
#include "mqueue.h"
#include "sys/msg.h"

#define KEY_BASE	100
#define PERM		0700

#define MAX_RECREATE_TRIES	500

#ifdef HAVE_MSGBUF
#	define V_MSGBUF		msgbuf
#else
#	define V_MSGBUF		v_msgbuf
	struct v_msgbuf
	{
		long	mtype;
		char	mtext[1];
	};
#endif

MQUEUE::MQUEUE(key_t tkey, int autocreate)
{
	mkey=tkey;
	msgid=-1;
	this->autocreate=autocreate;

	create();
}

MQUEUE::~MQUEUE()
{
}

MQUEUE::MQUEUE(MQUEUE &d)
{
	ddd(d);
}

MQUEUE MQUEUE::operator=(MQUEUE &d)
{
	ddd(d);
	return (*this);
}

void MQUEUE::ddd(MQUEUE &d)
{
	mkey=d.mkey;
	autocreate=d.autocreate;
	msgid=-1;
	create();
}

void	MQUEUE::create()
{
	int	k;

//	if (msgid!=-1)
//		destroy();	// removed 9/8/2001

	switch(mkey)
	{
		case MQUEUE_KEY_UNIQUE:
			k=KEY_BASE-1;
			do {
				k++;
				msgid=msgget(k,IPC_CREAT | IPC_EXCL | PERM);
			} while (msgid<0);
			mkey=k;
			break;
		case MQUEUE_KEY_PRIVATE:
			mkey=IPC_PRIVATE;
			msgid=msgget(mkey,PERM);
			break;
		default:
			msgid=msgget(mkey,0);
			if (msgid==-1)	// it doesn't exist
			{
				msgid=msgget(mkey,IPC_CREAT | PERM);
			}
			break;
	}
}

void MQUEUE::destroy()
{
	int	ret;

	if (msgid!=-1)
	{
		ret=msgctl(msgid,IPC_RMID,NULL);
		msgid=-1;
	}
}

int MQUEUE::check()
{
	int		ret;
	int		done;
	msqid_ds	buf;

	if (msgid==-1) ret=0;
	else
	{
		do {
			done=1;
			ret=msgctl(msgid,IPC_STAT,&buf);
			if (ret==-1) ret=0;	// error
			else ret=1;
		} while (!done);
	}

	return(ret);
}

int MQUEUE::key()
{
	return(mkey);
}

void	MQUEUE::push(long type, void *buf, size_t buflen)
{
	V_MSGBUF	*b;
	void		*p;
	int		ret,done;

	if (msgid==-1) return;

	p=malloc(buflen+sizeof(long));
	b=(V_MSGBUF *)p;
	b->mtype=type;
	memcpy(b->mtext,buf,buflen);

	do {
		done=1;
		ret=msgsnd(msgid,b,buflen,0);
		if (ret==-1)
		{
			switch(errno)
			{
				case EINTR:
					done=1;
					break;
				case EINVAL:
				case EFAULT:
#ifdef HAVE_EIDRM
				case EIDRM:
#endif
					msgid=-1;
					if (autocreate)
					{
						create();
						if (autocreate<MAX_RECREATE_TRIES)
						{
							autocreate++;
							push(type,buf,buflen);
						}
						autocreate=1;
					}
					break;
			}
		}
		if ((ret==-1) && (errno==EINTR)) done=0;	//retry
	} while (!done);

	free(p);
}

void	*MQUEUE::pop(size_t len,int block)
{
	return(pop(0,len,block));
}

void	*MQUEUE::pop(long type, size_t len, int block)
{
	int	ret,done;
	char	*p;
	V_MSGBUF	*b;

	p=(char *)malloc(len+sizeof(long));
	b=(V_MSGBUF *)p;
	do {
		done=1;
		ret=msgrcv(msgid,b,len,type,(block ? 0 : IPC_NOWAIT));
		if ((ret==-1) && (errno==EINTR)) done=0;	//retry
	} while (!done);
	if (ret==-1) { free(p); p=NULL; }
	else
	{
		memmove(p,p+sizeof(long),len);	//we dont need b->type any more
		p=(char *)realloc(p,len);	// free the extra bytes
	}

	return(p);
}



