/****************************************/
/*  Byte Queue                          */
/****************************************/
/* Start      : 23/12/2000              */
/* Program ID :                         */
/* Version    : 1.00                    */
/* End        :                         */
/* Last Update: 10/04/2004              */
/* Comments   : FIFO                    */
/*                                      */
/****************************************/

#include "bytequeue.h"

#include "v/minmax.h"
#include "string.h"

#include "v/mem/dlist3.cc"

#define BQ_BLOCKSIZE	8192 //192

ByteQueue::ByteQueue()
{
	begin=(char *)malloc(BQ_BLOCKSIZE);
	end=NULL;
	buflen=0;
	beg_ofs=0;
	end_data=0;
}

ByteQueue::~ByteQueue()
{
	clearall();
}

ByteQueue::ByteQueue(ByteQueue &d)
{
	begin=NULL;
	end=NULL;
	buflen=0;
	beg_ofs=0;
	end_data=0;
	ddd(d);
}

ByteQueue ByteQueue::operator=(ByteQueue &d)
{
	ddd(d);
	return (*this);
}

/*
	Try to avoid as many mallocs as possible
 */
void ByteQueue::ddd(ByteQueue &d)
{
	char	**p,**p2;
	char	*pp;
	int	domalloc;

	// If we have a begin block, then use it
	if (begin==NULL)
		begin=(char *)malloc(BQ_BLOCKSIZE);

	beg_ofs=d.beg_ofs;
	memcpy(begin+beg_ofs, d.begin+beg_ofs, BQ_BLOCKSIZE-beg_ofs);

	// Same for end
	// If not use a block from middle
	if (d.end==NULL)
	{
		if (end!=NULL)
			middle.PushL(end);
		end=NULL;
		end_data=0;
	} else
	{
		if (end==NULL)
		{
			p=middle.Pop();
			if (p==NULL)
				end=(char *)malloc(BQ_BLOCKSIZE);
			else
				end=*p;

			memcpy(end, d.end, d.end_data);
			end_data=d.end_data;
		}
	}

	// Copy middle using as many existing blocks as possible
	d.middle.Rewind();
	domalloc=0;
	middle.Rewind();
	while ((p=d.middle.Get_Next())!=NULL)
	{
		if (!domalloc)
		{
			p2=middle.Get_Next();
			if (p2==NULL)
				domalloc=1;
			else
			{
				memcpy(*p2,*p,BQ_BLOCKSIZE);
			}
		}
		if (domalloc)
		{
			pp=(char *)malloc(BQ_BLOCKSIZE);
			memcpy(pp,*p,BQ_BLOCKSIZE);
			middle.PushL(pp);
		}
	}
	// If there are more elements than in d free them
	if (!domalloc)
	{
		while (middle.Count>d.middle.Count)
		{
			p=middle.PopL();
			free(*p);
		}
	}

	buflen=d.buflen;
}

/*
	Free all memory. Even begin.
 */
void	ByteQueue::clearall()
{
	Clear();

	if (begin!=NULL) free(begin);
	begin=NULL;

}

/*
	Clear queue
 */
void	ByteQueue::Clear()
{
	char	**p;

	if (end!=NULL) free(end);
	end=NULL;
	end_data=0;
	buflen=beg_ofs=0;

	middle.Rewind();

	while ((p=middle.Get_Next())!=NULL)
	{
		free(*p);
	}

	middle.ClearList();
}


/*
	Append bytes to begin node.
	Set appended to number of bytes appended.
	return pointer to next byte to be added to queue or NULL
	update buflen
*/
const char *ByteQueue::append_buf_begin(const char *buf, size_t len, size_t *appended)
{
	size_t		n;
	int		cnt;
	const char	*p;

	p=buf;

	n=beg_ofs+buflen;
	if (n<BQ_BLOCKSIZE)	// if there is free space in first block
	{
		cnt=MIN(BQ_BLOCKSIZE-n,len);

		memcpy(begin+n, buf, cnt);

		*appended=cnt;
	} else
	{
		*appended=0;
	}

	p+=*appended;
	buflen+=*appended;
	if (p>=buf+len)
		p=NULL;

	return(p);
}

/*
	Fill as many full blocks as possible
	Set appended to number of bytes appended
	return as append_buf_begin()
	update buflen
*/
const char *ByteQueue::append_buf_middle(const char *buf, size_t len, size_t *appended)
{
	const char	*p;
	char		*pp;
	size_t		len2;

	len2=len;
	p=buf;
	*appended=0;

	while (len2>=BQ_BLOCKSIZE)
	{
		pp=(char *)malloc(BQ_BLOCKSIZE);
		memcpy(pp,p,BQ_BLOCKSIZE);
		len2-=BQ_BLOCKSIZE;
		p+=BQ_BLOCKSIZE;
		middle.PushL(pp);
		*appended+=BQ_BLOCKSIZE;
	}

	buflen+=*appended;
	if (p>=buf+len)
		p=NULL;

	return(p);
}

/*
	Same as append_buf_begin
	if end buffer fills, move it to middle and set end==NULL
*/
const char *ByteQueue::append_buf_end(const char *buf, size_t len, size_t *appended)
{
	size_t		n;
	size_t		cnt;
	const char	*p;

	p=buf;

	n=end_data;
	if (n<BQ_BLOCKSIZE)	// if there is free space in first block
	{
		cnt=MIN(BQ_BLOCKSIZE-n,len);

		memcpy(end+n, buf, cnt);

		end_data+=cnt;
		*appended=cnt;

		if (end_data==BQ_BLOCKSIZE)
		{
			middle.PushL(end);
			end=NULL;
			end_data=0;
		}
	} else
	{
		p=NULL;
		*appended=0;
	}

	p+=*appended;
	buflen+=*appended;
	if (p>=buf+len)
		p=NULL;

	return(p);
}

void	ByteQueue::create_new_end(const char *buf, size_t len)
{
	end=(char *)malloc(BQ_BLOCKSIZE);
	memcpy(end,buf,len);
	end_data=len;
	buflen+=len;
}

void	ByteQueue::append_buf(const char *buf, size_t len)
{
	const char	*p;
	size_t		appended=0;
	size_t		len2;

	len2=len;
	if (end==NULL)
	{
		// This will not hurt if there is no space in begin
		p=append_buf_begin(buf, len2, &appended);
	} else
	{
		p=append_buf_end(buf, len2, &appended);
	}

	len2-=appended;

	if (p!=NULL)
	{
		p=append_buf_middle(p, len2, &appended);
	}

	len2-=appended;

	if (p!=NULL)
	{
		create_new_end(p, len2);
	}
}

/*
	Store up to maxlen bytes in buf starting from the beggining
*/
size_t	ByteQueue::read_start(char *buf, size_t maxlen)
{
	size_t	toread,ret;
	size_t	n;
	char	*p;
	char	**pcurbuf;

	ret=toread=MIN(maxlen,buflen);
	p=buf;

	n=MIN(toread,BQ_BLOCKSIZE-beg_ofs);
	memcpy(p,begin+beg_ofs,n);
	toread-=n;
	p+=n;

	middle.Rewind();

	// Get the whole blocks
	while (((pcurbuf=middle.Get_Next())!=NULL) && (toread>=BQ_BLOCKSIZE))
	{
		memcpy(p,*pcurbuf,BQ_BLOCKSIZE);
		toread-=BQ_BLOCKSIZE;
		p+=BQ_BLOCKSIZE;
	}

	// If there are more data to read
	if (toread>0)
	{
		if (pcurbuf==NULL)	// now we read the end
		{
			memcpy(p,end,toread);
		} else	// or the rest of the last ecnountered full block
		{
			memcpy(p,*pcurbuf,toread);
		}
	}

	return(ret);
}

size_t	ByteQueue::discard_start(size_t maxlen)
{
	size_t	todiscard, ret;
	char	**p;
	size_t	n;

	ret=todiscard=MIN(maxlen,buflen);

	n=BQ_BLOCKSIZE-beg_ofs;
	if (todiscard<n)
	{
		beg_ofs+=todiscard;
		todiscard=0;
	} else if (todiscard==n)
	{
		free(begin);
		p=middle.Pop();
		if (p==NULL)
		{
			begin=end;
			end=NULL;
			end_data=0;
		} else
		{
			begin=*p;
		}
		beg_ofs=0;
	} else
	{
		beg_ofs=0;
		todiscard-=n;
		free(begin);
		while (((p=middle.Pop())!=NULL) && (todiscard>=BQ_BLOCKSIZE))
		{
			free(*p);
			todiscard-=BQ_BLOCKSIZE;
		}
		if (p==NULL)
		{
			if (todiscard==end_data)
				beg_ofs=0;
			else
				beg_ofs=todiscard;
			begin=end;
			end=NULL;
			end_data=0;
		} else
		{
			begin=*p;
			beg_ofs=todiscard;
		}
	}

	buflen-=ret;

	if (begin==NULL)
	{
		begin=(char *)malloc(BQ_BLOCKSIZE);
		beg_ofs=0;
	} else if (buflen==0)
		beg_ofs=0;

	return(ret);
}

void ByteQueue::Push(const char *buf, size_t len)
{
	append_buf(buf,len);
}

size_t ByteQueue::PopNoErase(char *buf, size_t maxlen)
{
	return(read_start(buf,maxlen));
}

size_t ByteQueue::Discard(size_t len)
{
	return(discard_start(len));
}

size_t ByteQueue::Pop(char *buf, size_t maxlen)
{
	size_t	l;

	l=read_start(buf,maxlen);
	discard_start(l);

	return(l);
}

void ByteQueue::UnPop(const char *buf, size_t len)
{
	size_t		toadd,added;
	char		*p;
	const char	*p2;

	toadd=len;
	added=0;

	if (buflen==0)
	{
		append_buf(buf,len);
		return;
	}

	if (buflen<BQ_BLOCKSIZE-beg_ofs)	// all data in begin - begin not full at the end
	{
		memmove(begin+BQ_BLOCKSIZE-buflen, begin+beg_ofs, buflen);
		beg_ofs=BQ_BLOCKSIZE-buflen;
	}
	if (toadd<=beg_ofs)
	{
		beg_ofs-=toadd;
		memcpy(begin+beg_ofs,buf,toadd);
	} else
	{
		memcpy(begin,buf+len-beg_ofs,beg_ofs);
		middle.Push(begin);
		added=beg_ofs;
		toadd-=beg_ofs;
		p2=buf+len-added;

		while (toadd>BQ_BLOCKSIZE)
		{
			p=(char *)malloc(BQ_BLOCKSIZE);
			p2-=BQ_BLOCKSIZE;
			memcpy(p,p2,BQ_BLOCKSIZE);
			toadd-=BQ_BLOCKSIZE;
		}
		begin=(char *)malloc(BQ_BLOCKSIZE);
		beg_ofs=BQ_BLOCKSIZE-toadd;
		memcpy(begin+beg_ofs,p2,toadd);
	}
	buflen+=len;
}

size_t ByteQueue::Size()
{
	return(buflen);
}

/*
 * Find the first line and return its size
 * or -1 if no line was available.
 *
 * size does not include \n \r
 */
long	ByteQueue::get_line_sz()
{
	char	*p, **pp;
	int	has_nl=0;
	long	remain;
	long	sz;
	int	maxsz;

	// Scan for \n\r
	p=begin+beg_ofs;

	sz=0;

	maxsz=MIN((BQ_BLOCKSIZE-beg_ofs), buflen);

	while (p<(begin+beg_ofs+maxsz) && *p!='\n' && *p!='\r')	// scan first block
		p++;

	// sz is the number of bytes without the first \n or \r
	sz=p-begin-beg_ofs;

	// end of begin block
	// try middles
	if (p==(begin+maxsz+beg_ofs))
	{
		middle.Rewind();

		while (!has_nl && ((pp=middle.Get_Next())!=NULL))
		{
			p=*pp;
			while (p<(*pp+BQ_BLOCKSIZE) && *p!='\n' && *p!='\r')
				p++;

			if (p<(*pp+BQ_BLOCKSIZE))
			{
				has_nl=1;
				sz+=p-*pp;
			} else sz+=BQ_BLOCKSIZE;
		}
	} else has_nl=1;

	if (!has_nl && end!=NULL)
	{
		p=end;

		remain=buflen-sz;
		while (p<(end+remain) && *p!='\n' && *p!='\r')
			p++;

		if (p<(end+remain))
		{
			sz+=p-end;
			has_nl=1;
		} else sz+=remain;
	}

	if (!has_nl) return(-1);

	return(sz);
}

/*
 * Get a line from the fifo
 * Store the line in line
 * line does not include trailing \n\r
 * Return size of line or -1 if no line was found
 */
long	ByteQueue::PopLineNoErase(LONGLINE &line)
{
	char	*p;
	long	sz;

	sz=get_line_sz();

	if (sz<0) return(-1);

	p=(char *)malloc(sz+1);

	read_start(p,sz);
	p[sz]=0;

	line=p;

	free(p);

	return(sz);
}

/*
 * Discard a line including \n\r
 *
 * Return number of discarded bytes (not including \n\r)
 */
long	ByteQueue::DiscardLine()
{
	long	sz;
	char	*p, ch;

	// Discard the line
	sz=get_line_sz();

	if (sz==-1) return(-1);

	discard_start(sz);

	// Remember the next character (\n or \r)
	p=begin+beg_ofs;
	ch=*p;

	// Discard it
	discard_start(1);

	// Now discard the next character if it is a \n\r or \r\n sequence
	// but not if it is \r\r or \n\n
	p=begin+beg_ofs;
	if ((*p=='\n' || *p=='\r') && *p!=ch)
		discard_start(1);

	return(sz);
}

/*
 * Return a line just like PopLineNoErase but also discard it
 *
 * Return size of line without the \n\r
 */
long	ByteQueue::PopLine(LONGLINE &line)
{
	long	ret;

	ret=PopLineNoErase(line);

	if (ret==-1) return(ret);

	DiscardLine();

	return(ret);
}

