/****************************************/
/*  Long line class (header)            */
/****************************************/
/* Start      : 12/8/98                 */
/* Program ID :                         */
/* Version    : 1.04                    */
/* End        :                         */
/* Last Update: 16/1/2002               */
/* Comments   : Class for very long     */
/*              lines with dynamic      */
/*              memory allocation       */
/****************************************/

/* Long string is a nice name too */
/* The allocated memory will be automaticaly resized */
/* for more or less memory.. As needed.*/

/*
 16/1/2002	v1.04:	Added ChopCRLF() and ChopSpaces() at last^2
 4/1/2000	v1.03:	Added void LONGLINE::operator--(int n)l at last.
 24/7/99	v1.02:	LONGLINE p=NULL is the same with LONGLINE p=""
			The constructor sets LONGLINE::p to point to "", instead of being NULL
			Added int LONGLINE::operator!=(LONGLINE &p2);
			Added int LONGLINE::operator!=(char *st);
 9/4/99		v1.01:	Added char *LONGLINE::operator+=(long l);
 */

#ifndef _VLIB_V_MEM_LONGLINE_H
#define _VLIB_V_MEM_LONGLINE_H

#include "string.h"
#include "stdlib.h"
#include "v/minmax.h"
#include "ctype.h"

class LONGLINE
{
 private:
 protected:
	long	size;  /* Size of allocated memory, in blocks */
 public:
	size_t	len;  /* Line length. (like strlen(LONGLINE)) (without the trailing 0)*/
	char	*p;
 private:
 protected:
	void	Resize		(long n); /* n = New size in blocks */
	size_t	CalcNewSize	(size_t n);  /* Find the apropriate size in blocks for n bytes */
	void	Init		();
 public:
		LONGLINE	();
		~LONGLINE	();
		LONGLINE	(const char *str);
		LONGLINE	(const LONGLINE &p1);
	char	*Strcat		(const char *src);
	char	*Strcat		(const LONGLINE& p2);
	char	*Strcpy		(const char *src);
	char	*Strcpy		(const LONGLINE& p2);
	char	*Strncat	(const char *src, size_t s);
	char	*Strncat	(const LONGLINE& p2, size_t s);
	char	*Strncpy	(const char *src, size_t s);
	char	*Strncpy	(const LONGLINE& p2, size_t s);
	int	Strcmp		(const LONGLINE& p2) const;
	int	Strcmp		(const char *st) const;
	char	*operator+=	(const char *src);
	char	*operator+=	(const LONGLINE& p2);
	char	*operator+=	(char c);
	char	*operator+=	(long l);

	char	*operator=	(const char *src);
	char	*operator=	(const LONGLINE& p2);

	LONGLINE operator+	(const char *src);
	LONGLINE operator+	(const LONGLINE& p2);
	LONGLINE operator+	(char c);

	void	operator--	();  //truncate the last character;
	void	operator--	(int n);  //truncate the last character;

	char	operator[]	(size_t s);

	int	operator==	(const LONGLINE& p2) const;
	int	operator==	(const char *st) const;

	int	operator!=	(const LONGLINE& p2) const;
	int	operator!=	(const char *st) const;

	void	Setsize		(size_t n);  /* Set the size to n bytes (at_least)*/
	void	Set		(size_t pos, char c);

	void	Lowercase	();
	void	Uppercase	();
	void	ChopCRLF	();	// Cut the trailing CRLFs
	void	ChopSpaces	();	// Cut all the trailing spaces, acordint o isspace() CR,LF,SPACE,TAB,VTAB,FF
};

class LONGMEM //: public LONGLINE
{
 private:
 	long	size;	// Allocated memory in blocks
 public:
 	char	*p;
 	size_t	len;	// Last used byte (with set)
 private:
	void	Init		();
	void	Resize		(long l);
	long	CalcNewSize	(size_t n);
 public:

		LONGMEM		();
		~LONGMEM	();
		LONGMEM		(const void *src,size_t s);
		LONGMEM		(LONGMEM& p2);
	int	operator==	(LONGMEM& p2);
	int	operator==	(const void *);
	LONGMEM	operator+	(LONGMEM& p2);
	LONGMEM	operator+	(char c);
	void	*operator+=	(char c);
	void	*operator+=	(LONGMEM& p2);
	void	*operator=	(LONGMEM& p2);
	void	operator--	(); // Truncate the last character;

	void	*Memcpy		(const void *src, size_t s);

	void	*Memcpy		(LONGMEM &p2);
	void	*Memcpy		(LONGMEM &p2, size_t s);
	void	*Memcat		(const void *src, size_t s);
	void	*Memcat		(LONGMEM &p2);
	void	*Memcat		(LONGMEM &p2, size_t s);
	int	Memcmp		(const void *src);
	int	Memcmp		(const void *src, size_t s);
	int	Memcmp		(LONGMEM &p2);
	void	Set		(size_t pos, char c);
	void	Empty		();	// Free all the memory
	char	&operator[]	(size_t pos);
	void	Setsize		(size_t n);
};

#ifndef _VLIB_V_MEM_LONGLINE_CC
extern "C++" {
#endif

/******************* NON memember functions *******************/

char *strcat(LONGLINE& p, const char *src);
char *strcat(LONGLINE& p1,const LONGLINE& p2);
char *strcpy(LONGLINE& p, const char *src);
char *strcpy(LONGLINE& p1,const LONGLINE& p2);
char *strncat(LONGLINE& p, const char *src, size_t s);
char *strncat(LONGLINE& p1,const LONGLINE& p2, size_t s);
char *strncpy(LONGLINE& p, const char *src, size_t s);
char *strncpy(LONGLINE& p1,const LONGLINE& p2, size_t s);
size_t strlen(const LONGLINE& p);
/*operator=(char *s, LONGLINE& p);*/


void *memcpy(LONGMEM& p1, void *p2, size_t s);
void *memcpy(LONGMEM&, void *, size_t);
void *memcpy(LONGMEM& p1, LONGMEM& p2);
void *memcpy(LONGMEM& p1, LONGMEM& p2, size_t s);
void *memcat(LONGMEM& p1, void *p2, size_t s);
void *memcat(LONGMEM& p1, LONGMEM& p2);
void *memcat(LONGMEM& p1, LONGMEM& p2, size_t s);
int memcmp(LONGMEM& p1, LONGMEM& p2);
int memcmp(LONGMEM& p1, void *p2, size_t s);
int memcmp(void *p1, LONGMEM& p2, size_t s);

/**************************************************************/

#ifndef _VLIB_V_MEM_LONGLINE_CC
}
#endif

// #define _NO_MEMCPY
#ifdef _NO_MEMCPY
extern "C" {

void *memcpy(const void *,const void*,size_t s);
int memcmp(const void *,const void*,size_t s);

}
#endif
#endif
