/****************************************/
/*  Generalized config file (header)    */
/****************************************/
/* Start      : 3/1/2000                */
/* Program ID :                         */
/* Version    : 1.03                    */
/* End        :                         */
/* Last Update: 2/8/2001                */
/* Comments   :                         */
/*                                      */
/****************************************/

#ifndef _VLIB_V_MISC_CONFIGFILE_H
#define _VLIB_V_MISC_CONFIGFILE_H

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "v/misc/keytable2.h"
#include "v/string/parse.h"

/*
	Read variables from a config file (class Read_Config)
	and store them to a VARTABLE (defined as KEYTABLE<Variable>)
	Each variable, may have as many attributes as needed.	
	The whole config file and the whole text file is loaded into
	memory, and for an instance, the text file is hold twice in
	the memory, so it may not be used for large files.

	This was originaly create to parse html template pages, which
	explains the name of the class Html_Parser

	The format of the config file is:
	[Variable name]
	ATTRIBUTE=VALUE
	ATTRIBUTE={ VALUE }
	ATTRIBUTE={ VALUE
	VALUE}

	[Variable name]
	.
	.
	.

	When using ATTRIBUTE=VALUE,
	the spaces at the begginig and the end of VALUE are truncated.
	This means that
	TEXT = MY_TEXT   (with a trailing space)
	is the same as
	TEXT=MY_TEXT

	If this is not wanded, then {} may be used. In this case, the EXACTLY
	text contained in {} will be used.
	TEXT= { MY_TEXT }

	If {} are used, multiple lines may be specifed. The { MUST be after the =
	(in the same line).

	to use } or { in the attribute's value use \.
	ATTRIBUTE=\{TEST}
	(This is valid, bacause no { was found, so a } was not expected, but using \
	will not cause a problem.
	ATTRIBUTE={TEST\}TEST}
	To use \ in the text, prepend it with \.
	ATTRIBUTE=\\test\\
*/


class Variable
{
 private:
 public:
 	LONGLINE		name;	// The name of the variable
 	KEYTABLE<LONGLINE>	attr;	// The attributes (if any)
 private:
	void ddd(Variable &d);
 public:
	Variable();
	~Variable();
	Variable(Variable &d);
	Variable operator=(Variable &d);
	void clear();
};

typedef KEYTABLE<Variable> VARTABLE;
typedef VARTABLE *pVARTABLE;

//#define VARTABLE	KEYTABLE<Variable>
//#define pVARTABLE	VARTABLE *

class Read_Config
{
 private:
 	LONGLINE	conf;
 	VARTABLE	*vars;
 private:
 	void do_var(char *c);
 	void parse();
 	void create_line();
 public:
 	Read_Config(VARTABLE *vars);
 	~Read_Config();
 	int Read(const char *fn);			// Read a config file. Ret: 1: OK, 0:Error
 	int Write(const char *fn);			// Create a config file, Ret: 1:OK, 0:Error
};


#ifndef _VLIB_V_MISC_CONFIGFILE_CC
extern "C++" {
#endif
char	*ignore_SCRLF(char *c);			// pass through spaces, CRs, LFs
char	*get_block(char *c, LONGLINE& dest);	// get a block (all chars until EOL, or everything included in {}
int	load(const char *fn, LONGLINE& dest);		// Load a file in dest - Return: 1: OK. 0: Error opening file
int	save(const char *fn,const LONGLINE& src);       // Save src in fn - Ret: 1:OK 0:Error

#ifndef _VLIB_V_MISC_CONFIGFILE_CC
}
#endif


#endif
