/****************************************/
/*  KEYTABLE class                      */
/****************************************/
/* Start      : 6/1/2000                */
/* Program ID :                         */
/* Version    : 2.02                    */
/* End        :                         */
/* Last Update: 3/7/2001                */
/* Comments   : Table with string keys  */
/*              (dynamic table)         */
/****************************************/

#ifndef _VLIB_V_MISC_KEYTABLE2_CC
#define _VLIB_V_MISC_KEYTABLE2_CC

#include "v/lconfig.h"
#include "keytable2.h"
#include "v/mem/dlist3.cc"

#ifdef HAVE_STRING_H
#	include <string.h>
#else
#	include <strings.h>
#endif

#ifdef _KT_CASE_SENSITIVE
#	define	STRCOMP	strcmp
#else
#	ifdef HAVE_STRCASECMP
#		define	STRCOMP strcasecmp
#	else
#		define	STRCOMP stricmp
#	endif
#endif

template <class T, int HS, int MD> KEYTABLE2<T,HS,MD>::~KEYTABLE2()
{
	ClearAll();
}


template <class T, int HS, int MD> long KEYTABLE2<T,HS,MD>::Exist(const LONGLINE& s)
{
 long n=0;
 KTOBJ2<T,HS> *p;

 p=GetKTO(s);
 if (p!=NULL)
  n=p->ID;

 return(n);
}

template <class T, int HS, int MD> T *KEYTABLE2<T,HS,MD>::Get(const LONGLINE& str)
{
 KTOBJ2<T,HS> *p;
 T *t;

 t=NULL;

 p=GetKTO(str);
 if (p!=NULL)
  t=&(p->d);

 return(t);
}

template <class T, int HS, int MD> T *KEYTABLE2<T,HS,MD>::operator[](const LONGLINE &str)
{
 return(Get(str));
}

template <class T, int HS, int MD> T *KEYTABLE2<T,HS,MD>::operator[](const char *s)
{
 LONGLINE ll;
 ll=s;
 return(Get(ll));
}

template <class T, int HS, int MD> KEYTABLE2<T,HS,MD>::KEYTABLE2(KEYTABLE2<T,HS,MD> &p)
{
	Init();
	vvv(p);
}

template <class T, int HS, int MD> KEYTABLE2<T,HS,MD> KEYTABLE2<T,HS,MD>::operator=(KEYTABLE2<T,HS,MD>& p)
{
 	ClearAll();
	vvv(p);
	return(*this);
}

template <class T, int HS, int MD> KT_HASHBLOCK<T,HS> *KEYTABLE2<T,HS,MD>::GetHashNode(const char *name) //, KT_HASHBLOCK<T,HS> *cur, int curdep)
{
	KT_HASHBLOCK<T,HS>	*cur;
	int			n,OK=0;

	cur=&hash_head;
	lasthash=cur;
	for (n=0;(n<MD && !OK);n++)
	{
		if (!name[n])	// Target Reached
			OK=1;
		else
		{
			cur=cur->childs[name[n]/hbs];
			if (cur==NULL) OK=1;	// No more nodes exist
			else lasthash=cur;
		}
	}
	if (cur!=NULL)
		lasthash=cur;

	return(cur);
}

template <class T, int HS, int MD> void KEYTABLE2<T,HS,MD>::HashAdd(KTOBJ2<T,HS> *d)
{
	KT_HASHBLOCK<T,HS>	*p,*p2;
	KTOBJ2<T,HS>		*ktp;
	int			n;
	char			*name;

	name=d->str.p;

	p=GetHashNode(name);

	p=lasthash;
	for (n=p->level;(n<MD && name[n]);n++)	// Add the remaining nodes (if needed)
	{
		p2=new KT_HASHBLOCK<T,HS>;
		p2->parent=p;
		p->childs[name[n]/hbs]=p2;
		p2->level=n+1;
		p=p2;
	}

	// at this point, p is our node. just set it's value (if needed)
	ktp=p->p;
	d->ibprev=NULL;	// Add it at the start of the list
	d->ibnext=ktp;
	if (ktp!=NULL)
	{
		ktp->ibprev=d;
	}
	p->p=d;

	d->hblock=p;
}

/*
	Check if a node is needed. Delete it if not
*/
template <class T, int HS, int MD> void KEYTABLE2<T,HS,MD>::CheckAndDel(KT_HASHBLOCK<T,HS> *node)
{
	KT_HASHBLOCK<T,HS>	*parent;

	if (node==NULL || node==&hash_head)	// Do not remove the head :)
		return;

	if (node->p==NULL)	// if there are no data any more on this node..
	{			// consider removing it
		if (node->NoChilds())	// if there are no childs
		{
			parent=node->parent;
			parent->UnlinkChild(node);		// unlink it from the parent
			delete(node);				// and erase it
			CheckAndDel(parent);			// Now check if the parent is needed any more
		}
	}
}

/*
	Notify the hash table, that some data are going to be removed.
	Remove the pointer to that data. Set the pointer to another value if needed to keep consistency
	Check if the node is needed any more
*/
template <class T, int HS, int MD> void KEYTABLE2<T,HS,MD>::HashDel(KTOBJ2<T,HS> *d)
{
	KT_HASHBLOCK<T,HS>	*node;
	KTOBJ2<T,HS>		*p;

	node=d->hblock;
	p=d;
/*	if (p->ibprev!=NULL)
	{
		while (p->ibprev!=NULL)
			p=p->ibprev;
	} else
		p=p->ibnext;

	node->p=p;*/

	if (p->ibprev!=NULL)
	{
		p->ibprev->ibnext=p->ibnext;
	} else
	{
		node->p=p->ibnext;
	}

	if (p->ibnext!=NULL)
	{
		p->ibnext->ibprev=p->ibprev;
	}

	if (node->p==NULL)
		CheckAndDel(node);

/*	if (p!=NULL)
		p->ibprev=NULL;
	else
		CheckAndDel(node);*/
}

template <class T, int HS, int MD> KEYTABLE2<T,HS,MD>::KEYTABLE2()
{
	Init();
}

template <class T, int HS, int MD> void KEYTABLE2<T,HS,MD>::Init()
{
	NextID=1;
	lastsearch=NULL;
	hash_head.level=0;
	lasthash=&hash_head;
	head=tail=NULL;
	lastGetNext=head;
	hbs=256/HS;
	count=0;
}

template <class T, int HS, int MD> KTOBJ2<T,HS> *KEYTABLE2<T,HS,MD>::GetKTO(const LONGLINE& s)
{
	KTOBJ2<T,HS>		*p,*ret;
	KT_HASHBLOCK<T,HS>	*phb;
	const char		*st;

	p=lastsearch;
	ret=NULL;
	if ((p==NULL) || (STRCOMP(p->str.p,s.p)))	// If we are allready there, don't search again. (6/1/2000)
	{
		phb=GetHashNode(s.p);	// Get the node;
		if (phb==NULL)	p=NULL;	// No such node
		else
		{
			p=phb->p;
			if (p==NULL)	// Not found
				ret=NULL;
			else
			{
				st=s.p;
				while (p!=NULL && STRCOMP(st,p->str.p))
				{
						p=p->ibnext;
				}
				ret=p;
			}
		}
	} else ret=p;

	return(ret);
}

template <class T, int HS, int MD> void KEYTABLE2<T,HS,MD>::Del(const LONGLINE& str)
{
	KTOBJ2<T,HS>	*d,*p;

	d=GetKTO(str);
	if (d!=NULL)
	{
		HashDel(d);
		p=d->prev;
		if (p!=NULL)
		{
			p->next=d->next;
		} else
		{
			head=d->next;
		}
		p=d->next;
		if (p!=NULL)
		{
			p->prev=d->prev;
		} else
		{
			tail=d->prev;
		}
		lastGetNext=head;
//		lastsearch=NULL;	// 19/4/2001
		count--;
		delete(d);
	}
}

template <class T, int HS, int MD> void KEYTABLE2<T,HS,MD>::ClearAll()
{
	int		n;
	KTOBJ2<T,HS>	*p1,*p2;

	for (n=0;n<HS;n++)
	{
		DelTree(hash_head.childs[n]);
		hash_head.childs[n]=NULL;
	}
	hash_head.p=NULL;

	p1=head;
	while (p1!=NULL)
	{
		p2=p1->next;
		delete(p1);
		p1=p2;
	}

	count=0;
	lastsearch=NULL;
	head=NULL;
	tail=NULL;
	lastGetNext=head;
	lasthash=NULL;
	NextID=1;
}

template <class T, int HS, int MD> void KEYTABLE2<T,HS,MD>::DelTree(KT_HASHBLOCK<T,HS> *tree)
{
	int n;

	if (tree!=NULL)
	{
		for (n=0;n<HS;n++)
			DelTree(tree->childs[n]);
		delete(tree);
	}
}

template <class T, int HS, int MD> long KEYTABLE2<T,HS,MD>::AddSorted(const LONGLINE& str, T p)
{
	KTOBJ2<T,HS>		*pkto;
	long			ret;

	pkto=GetKTO(str);
	if (pkto==NULL)
	{
		pkto=new KTOBJ2<T,HS>;
		pkto->d=p;
		pkto->ID=NextID++;
		pkto->str=str;

		pkto->next=NULL;
		pkto->prev=tail;
		if (tail!=NULL)
		{
			tail->next=pkto;
		}
		tail=pkto;
		if (head==NULL)
			head=tail;

		HashAdd(pkto);
		count++;
		ret=NextID-1;
  	} else
	{
		pkto->d=p;
		ret=pkto->ID;
	}

	return(ret);
}

template <class T, int HS, int MD> long KEYTABLE2<T,HS,MD>::Add(const LONGLINE& str, T p)
{
	return(AddSorted(str,p));
}

template <class T, int HS, int MD> void KEYTABLE2<T,HS,MD>::vvv(KEYTABLE2<T,HS,MD>& d)
{
	KTOBJ2<T,HS>	*p;
	
	p=d.head;
	while (p!=NULL)
	{
		Add(p->str,p->d);
		p=p->next;
	}
}

template <class T, int HS, int MD> void KEYTABLE2<T,HS,MD>::ClearTable()
{
	ClearAll();
}

template <class T, int HS, int MD> void KEYTABLE2<T,HS,MD>::Rewind()
{
	lastGetNext=head;
}

template <class T, int HS, int MD> LONGLINE *KEYTABLE2<T,HS,MD>::Get_Next()
{
	KTOBJ2<T,HS>	*p;
	LONGLINE	*pll=NULL;

	p=lastGetNext;
	if (p!=NULL)
	{
		lastGetNext=lastGetNext->next;
		pll=&(p->str);
//		GetNextBuf=p->str;	//19/4/2001	// horible bug.. this will fail when passing more than one parameters with []
//		pll=&GetNextBuf;	//19/4/2001
	}
	
	return(pll);
}

template <class T, int HS, int MD> int KEYTABLE2<T,HS,MD>::isinsameblock(const char *st1, const char *st2)
{
	int	ret=1,n;
	
	for (n=0;(n<MD && st1[n] && st2[n] && ret);n++)
	{
		if (st1[n]/hbs!=st2[n]/hbs)
			ret=0;
	}

	return(ret);
}

#endif
