/****************************************/
/*  Dynamic Library Loading             */
/****************************************/
/* Start      : 20/4/2001               */
/* Program ID :                         */
/* Version    : 1.00                    */
/* End        :                         */
/* Last Update: 20/4/2001               */
/* Comments   :                         */
/*                                      */
/****************************************/

#include "libloader.h"
#include "v/mem/dlist3.cc"
#include "v/misc/keytable2.cc"
#include "v/lconfig.h"

LIBLOADER::LIBLOADER()
{
}

LIBLOADER::~LIBLOADER()
{
}

/*LIBLOADER::LIBLOADER(LIBLOADER &d)
{
	ddd(d);
}

LIBLOADER LIBLOADER::operator=(LIBLOADER &d)
{
	ddd(d);
	return (*this);
}

void	LIBLOADER::ddd(LIBLOADER &d)
{
}*/

int	LIBLOADER::loadlib(const char *libname, sym_resolve resolv)
{
	LIB	*p;
	void	*h;
	int	ret = LOADLIB_OK;

	h=dlopen(libname, ( resolv==lazy ? RTLD_LAZY : RTLD_NOW )| RTLD_GLOBAL);

	if (h==NULL)
	{
		ret=LOADLIB_ERROR;
	} else
	{
		p=new(LIB);
		p->handle=h;
		p->name=strdup(libname);
		Libs.PushL(p);
	}

	return(ret);
}

int	LIBLOADER::unloadlib(LIB *plib)
{
	int	ret=UNLOADLIB_OK;

	if (plib==NULL) ret=UNLOADLIB_ERROR;
	else
	{
		if ((dlclose(plib->handle))!=0)
			ret=UNLOADLIB_ERROR;
   		else
			DelLib(plib->name);
	}

	return(ret);
}

int	LIBLOADER::unloadlib(const char *libname)
{
	return(unloadlib(GetLib(libname)));
}

LIBLOADER::LIB	*LIBLOADER::GetLib(const char *libname)
{
	LIB	**p, *ret=NULL;
	int		OK=0;

	Libs.Rewind();

	while (!OK)
	{
		p=Libs.Get_Next();
		if (p==NULL) OK=1;
		else
		{
			if (!strcmp((*p)->name,libname))
			{
				OK=1;
				ret=*p;
			}
		}
	}

	return(ret);
}

void	LIBLOADER::DelLib(const char *libname)
{
	LIB	**p;
	int	OK=0;

	Libs.Rewind();
	while (!OK)
	{
		p=Libs.Get_Next();
		if (p==NULL) OK=1;
		else
		{
			if (!strcmp((*p)->name,libname))
			{
				OK=1;
				Libs.RemoveCurrent();
			}
		}
	}
}

/****************************************************************************************/
/************************************   LIBFUNCLIST   ***********************************/
/****************************************************************************************/

LIBFUNCLIST::LIBFUNCLIST()
{
}

LIBFUNCLIST::~LIBFUNCLIST()
{
	Clear();
}

LIBFUNCLIST::LIBFUNCLIST(LIBFUNCLIST &d)
{
	ddd(d);
}

LIBFUNCLIST LIBFUNCLIST::operator=(LIBFUNCLIST &d)
{
	ddd(d);
	return (*this);
}

void	LIBFUNCLIST::ddd(LIBFUNCLIST &d)
{
	funcs=d.funcs;
}

int	LIBFUNCLIST::addfunc(const char *funcname, LIBLOADER::LIB *plib)
{
	void		*f;
	int		ret=GETFUNC_OK;
	LIBFUNC		*plf;

	f=dlsym(plib->handle, funcname);
	if (f==NULL) ret=GETFUNC_ERROR;
	else
	{
		plf=new LIBFUNC;

		plf->func=f;
		plf->funcname=strdup(funcname);
		plf->lib=plib;

		funcs.Add(funcname, plf);
	}

	return(ret);
}

int	LIBFUNCLIST::delfunc(const char *funcname)
{
	LIBFUNC		**pplf;
	int		ret=GETFUNC_OK;

	pplf=funcs[funcname];
	if (pplf==NULL) ret=GETFUNC_ERROR;
	else
	{
		delete(*pplf);
		funcs.Del(funcname);
	}

	return(ret);
}

void	LIBFUNCLIST::Clear()
{
	LIBFUNC		**pplf;
	LONGLINE	*pl;

	funcs.Rewind();
	while ((pl=funcs.Get_Next())!=NULL)
	{
		pplf=funcs[*pl];
		delete(*pplf);
	}
	funcs.ClearTable();
}

void	*LIBFUNCLIST::getfunc(const char *funcname)
{
	void		*f=NULL;
	LIBFUNC		*plf;

	plf=lfgetfunc(funcname);

	if (plf!=NULL)
		f=plf->func;

  	return(f);
}

LIBFUNCLIST::LIBFUNC	*LIBFUNCLIST::lfgetfunc(const char *funcname)
{
	LIBFUNC		**pplf,*plf=NULL;

	pplf=funcs[funcname];
	if (pplf!=NULL)
		plf=*pplf;

	return(plf);
}

int	LIBFUNCLIST::exist(const char *funcname)
{
	return( (getfunc(funcname)==NULL ? 0 : 1) );
}
