/****************************************/
/*					*/
/****************************************/
/* Start      : 09/02/2002		*/
/* Program ID : 			*/
/* Version    : 1.00			*/
/* End        : 			*/
/* Last Update: 09/02/2002		*/
/* Comments   : 			*/
/*                                      */
/****************************************/

#include "scheduler.h"

#include "v/mem/dlist3.cc"

#include "limits.h"

template <class T> SCHEDULER<T>::SCHEDULER()
{
	next_run=INT_MAX;
}

template <class T> SCHEDULER<T>::~SCHEDULER()
{
}

template <class T> void SCHEDULER<T>::Add(time_t t, hook_func func, T *cls, int flags, int ID)
{
	SCH_ENTRY	s,*p;

	p=get(func,cls,ID);
	if (p!=NULL)
		return;

	s.ID=ID;
	s.func=func;
	s.cls=cls;
	s.next_run=time(NULL)+t;
	s.interval=t;
	s.flags=flags;

	hooks.PushL(s);

	if (s.next_run<this->next_run)
		this->next_run=s.next_run;
}

template <class T> typename SCHEDULER<T>::SCH_ENTRY *SCHEDULER<T>::get(hook_func func, T *cls, int ID)
{
	SCH_ENTRY	*p;

	hooks.Rewind();

	while ((p=hooks.Get_Next())!=NULL)
	{
		if ((p->func==func) && (p->cls==cls) && (p->ID==ID))
		{
			return(p);
		}
	}
	return(NULL);
}

template <class T> void SCHEDULER<T>::Cancel(hook_func func, T *cls, int ID)
{
	SCH_ENTRY	*p;

	p=get(func,cls,ID);
	if (p!=NULL)
		hooks.RemoveItem(p);

	this->next_run=CalcNextRun();
}

template <class T> time_t SCHEDULER<T>::Check()
{
	SCH_ENTRY	*p;
	time_t		now=time(NULL);
	hook_func	hf;
	T		*pcls;

	if (now<next_run)
		return(next_run);

	hooks.Rewind();
	while ((p=hooks.Get_Next())!=NULL)
	{
		if (p->next_run<=now)
		{
			pcls=p->cls;
			hf=p->func;
			if (p->flags & SCH_RESTART)
				p->next_run=now+p->interval;	// Reschedule it from now. not from the scheduled run
			else
			{
				hooks.RemoveCurrent();
				hooks.Rewind();
			}
			(pcls->*hf)();
		}
	}
	next_run=CalcNextRun();

	return(next_run);
}

template <class T> time_t SCHEDULER<T>::CalcNextRun()
{
	SCH_ENTRY	*p;
	time_t		now=time(NULL);
	time_t		next=INT_MAX;

	hooks.Rewind();
	while ((p=hooks.Get_Next())!=NULL)
	{
		if (p->next_run<=now)	// if this is for the past (or now)
			return(now);

		if (p->next_run<next)
			next=p->next_run;
	}

	return(next);
}

