/****************************************/
/*					*/
/****************************************/
/* Start      : 09/02/2002		*/
/* Program ID : 			*/
/* Version    : 1.00			*/
/* End        : 			*/
/* Last Update: 09/02/2002		*/
/* Comments   : 			*/
/*                                      */
/****************************************/

#ifndef _VLIB_V_MISC_SCHEDULER_H
#define _VLIB_V_MISC_SCHEDULER_H

#include "v/mem/dlist3.h"

#include "time.h"

#define	SCH_RESTART	1	// Reschedule it after interval

/*
	Scheduler.

	SCHEDULER<TEST_CLASS>	scheduler.
	TEST_CLASS		tst;

	scheduler.Add(10,&TEST_CLASS::func,&tst);	// will schedule TEST_CLASS::func to be called in 10 seconds

	scheduler.Add(10,&TEST_CLASS::func,*tst,SCH_RESTART);	// function will be called every 10 seconds

	scheduler.Add(10,&TEST_CLASS::func,&test,0,1);
	scheduler.Add(11,&TEST_CLASS::func,&test,0,2);	// func will be called after 10 seconds AND after 11 seconds

	ID is used to distinguish multiple schedules of the same function (of the same class). Default is 0.

	in th 3rd example above, using:
	scheduler.del(&TEST_CLASS::func,&test,2);	// will cancel the 2nd entry
*/

template <class T> class SCHEDULER
{
 private:
 	typedef void (T::*hook_func)();
	struct SCH_ENTRY
	{
		int		ID;
		hook_func	func;
		T		*cls;
		time_t		next_run;
		time_t		interval;
		long		flags;
	};
	DL_List<SCH_ENTRY>	hooks;
	time_t			next_run;
 private:
 	SCH_ENTRY	*get	(hook_func func, T *cls, int ID=0);
 public:
 		SCHEDULER	();
		~SCHEDULER	();
	void	Add		(time_t t, hook_func func, T *cls, int flags=0, int ID=0);	// t specifies the seconds from now
												// if it exists, nothing is done
	void	Cancel		(hook_func func, T *cls, int ID=0);
	time_t	Check		();	// Calls the wanted functions. Returns the time of the next run
	time_t	CalcNextRun	();	// This will return the time of next run
};

#endif

