/****************************************/
/*					*/
/****************************************/
/* Start      : 14/05/2002		*/
/* Program ID : 			*/
/* Version    : 1.00			*/
/* End        : 			*/
/* Last Update: 14/05/2002		*/
/* Comments   : 			*/
/*                                      */
/****************************************/

#define SIMPLECONFIG_DEFINESIMPLE

#include "simpleconfigfile.h"

#include "v/mem/dlist3.cc"
#include "v/mem/strlist.h"
#include "v/socket/ipaddr.h"
#include "v/string/parse.h"

#include "errno.h"

/*
	Internal flags
*/
#define	CFG_FL_SET	0x010000	// If this value is set once (to check for rehashing)
#define CFG_FL_HASVALUE	0x020000	// If there is a value set (to check for EMPTY)

SIMPLE_CONFIG::SIMPLE_CONFIG(CFG_VAR *vars)
{
	this->vars=vars;
	ClearConfig();
}

SIMPLE_CONFIG::~SIMPLE_CONFIG()
{
}

/*
	Clear configuration table
*/
void SIMPLE_CONFIG::ClearConfig()
{
	int		n;
	STRLIST		*pstrlist;

	for (n=0;vars[n].name!=NULL;n++)
	{
		if (!(vars[n].flags & NOINIT))
		{
			if (vars[n].dflt!=NULL)
			{
				ConfAdd(vars[n].name,vars[n].dflt);
			} else
			{
				switch(vars[n].type)
				{
					case TYPE_U32:
					case TYPE_U16:
					case TYPE_DBL:
						ConfAdd(vars[n].name,"0");
						break;
					case TYPE_IP_LL:
					case TYPE_IP_CHAR_P:
					case TYPE_IP_U32:
						ConfAdd(vars[n].name,"0.0.0.0");
						break;
					case TYPE_LL:
					case TYPE_CHAR_P:
						ConfAdd(vars[n].name,"");
						break;
					case TYPE_FUNC:
						ConfAdd(vars[n].name,"");
						break;
					case TYPE_STRLIST:
						pstrlist=(STRLIST *)vars[n].p;
						pstrlist->Clear();
						break;
					case TYPE_BOOL:
						ConfAdd(vars[n].name,"False");
				}
			}
			if (vars[n].flags & CFG_FL_SET)
			{
				vars[n].flags &= ~CFG_FL_SET;
			}
		}
	}
}

void	SIMPLE_CONFIG::CheckOpt(const char *var, const char *value)
{
	int		n;
	CFG_VAR		*pvar;
	CFG_VAR_OPT	*popt;
	LONGLINE	err1,err2;

	err1="Configuration error in ";
	err1+=this->fn;
	err1+=": ";
	err1+=var;
	err1+=": ";
		
	pvar=NULL;
	for (n=0; (vars[n].name!=NULL && pvar==NULL) ; n++)
	{
		if ((((vars[n].flags & CASE)) && (!strcmp(vars[n].name,var))) ||
			(!strcasecmp(vars[n].name,var)))
		{
			pvar=&(vars[n]);
		}
	}
	
	if (pvar!=NULL)
	{
		popt=pvar->opt;
		
		while (popt!=NULL)
		{
			switch(popt->opt)
			{
				case CFG_VAR_OPT_SET:
					{
						int	ok=0;
						char	**p;
						
						// Check if value is in the set
						p=(char **)popt->data;
						for (n=0;( n<popt->size && !ok );n++)
						{
							if (popt->flags & CFG_VAR_OPT_SET_FL_CASE)
							{
								if (!strcmp(p[n],value))
									ok=1;
							} else
							{
								if (!strcasecmp(p[n],value))
									ok=1;
							}
						}
						if (!ok)
						{
							err2=err1;
							err2+="Value must be one of ";
							for (n=0;n<popt->size;n++)
							{
								err2+=p[n];
								err2+=",";
							}
							if (n>0)
								err2--;
							Errors.PushL(err2);
						}
					}
					break;
			}
			popt=popt->next;
		}
	}
}

/*
	This must not be called with value==NULL, unless type is TYPE_CHAR_P
*/
void	SIMPLE_CONFIG::ConfAdd(const char *var, const char *value)
{
	u_32		*pu32;
	u_16		*pu16;
	double		*pdbl;
	cfg_hookfunc	pfunc;
	int		n;
	LONGLINE	err;

	err="Configuration error in ";
	err+=this->fn;
	err+=": ";
	err+=var;
	err+=": ";

	for (n=0;vars[n].name!=NULL;n++)
	{
		if (!strcasecmp(vars[n].name,var))
		{
			if ((!(vars[n].flags & CASE)) || (!strcmp(vars[n].name,var)))
			{
				if ((!(vars[n].flags & CFG_FL_SET)) || (vars[n].flags & REHASH))
				{
					switch(vars[n].type)
					{
						case TYPE_U32:
							pu32=(u_32 *)vars[n].p;
							*pu32=(u_32)atol(value);
							break;
						case TYPE_U16:
							pu16=(u_16 *)vars[n].p;
							*pu16=(u_16)atol(value);
							break;
						case TYPE_LL:
						case TYPE_IP_LL:
							*((LONGLINE *)vars[n].p)=value;
							break;
						case TYPE_CHAR_P:
						case TYPE_IP_CHAR_P:
							if ((vars[n].flags & CFG_FL_SET) && (*(char **)vars[n].p!=NULL))	// if there is an older value, free it
								free(*(char**)vars[n].p);

							*((char **)vars[n].p)=strdup((value==NULL ? "" : value));

							break;
						case TYPE_IP_U32:
							{
								IPADDR	ip;
								ip=value;
								pu32=(u_32 *)vars[n].p;
								*pu32=ip.ipaddr.s_addr;
							}
							break;
						case TYPE_DBL:
							pdbl=(double *)vars[n].p;
							*pdbl=(double)atof(value);
							break;
						case TYPE_FUNC:
							{
								int		m;
								char		*buf=NULL;

								pfunc=(cfg_hookfunc)vars[n].p;
								m=pfunc(&(vars[n]), value, &buf);
								switch(m)
								{
									case RET_WARNING:
										err+=buf;
										Warnings.PushL(err);
										if (buf!=NULL) free(buf);
										return;
										break;
									case RET_ERROR:
										err+=buf;
										Errors.PushL(err);
										if (buf!=NULL) free(buf);
										return;
										break;
								}
							}
							break;
						case TYPE_STRLIST:
							{
								PARSE	pars;
								char	*p;
								int	m;

								STRLIST	*pstrlist;

								pars.AddSep(' ');
								pars.AddSep(',');
								pars.AddSep('\n');
								pars.AddSep('\r');
								pars.AddSep('\t');
								pars.flags=PARSEFLAG_IGNOREEMPTY;
								pars.Set(value);

								pstrlist=(STRLIST *)vars[n].p;
								pstrlist->Clear();
								m=0;
								while ((p=pars[m++])!=NULL)
								{
									pstrlist->Add(p);
								}
							}
							break;
						case TYPE_BOOL:
							{
								char	c;
								char	*p;

								if ((!strcasecmp(value,"true"))
									|| (!strcasecmp(value,"on"))
									|| (!strcasecmp(value,"yes"))
									|| (atol(value)>0))
								{
									c=1;
								} else if ((!strcasecmp(value,"false"))
									|| (!strcasecmp(value,"off"))
									|| (!strcasecmp(value,"no"))
									|| (atol(value)==0))
								{
									c=0;
								} else
								{
									err+="Value must be one of true,on,yes,false,off,no or a non-negative integer";
									Errors.PushL(err);
									return;
								}
								p=(char *)vars[n].p;
								*p=c;
							}
							break;
					}
					vars[n].flags|=CFG_FL_SET;
					vars[n].flags|=CFG_FL_HASVALUE;
				}
			}
			return;
		}
	}

	err+="Unknown variable (value: ";
	err+=value;
	err+=")";

	Warnings.PushL(err);
}

void	SIMPLE_CONFIG::CheckConfig()
{
	int		n;
	int		error=0;
	LONGLINE	err1,err2,err3;

	err1="Configuration error in ";
	err1+=this->fn;
	err1+=": ";

	for (n=0;vars[n].name!=NULL;n++)
	{
		err2=err1;
		err2+=vars[n].name;
		err2+=": ";
		error=0;

		if (!(vars[n].flags & (EMPTY | CFG_FL_HASVALUE)))
		{
			err2+="A value is required";
			Errors.PushL(err2);
		} else if (!(vars[n].flags & ZERO))
		{
			switch(vars[n].type)
			{
				case TYPE_U32:
				case TYPE_IP_U32:
					if (*(u_32 *)vars[n].p==0) error=1;
					break;
				case TYPE_U16:
					if (*(u_16 *)vars[n].p==0) error=1;
					break;
				case TYPE_LL:
				case TYPE_IP_LL:
					if (*(LONGLINE *)vars[n].p=="") error=2;
					break;
				case TYPE_CHAR_P:
				case TYPE_IP_CHAR_P:
					if ((*(char **)vars[n].p==NULL) ||
						(**(char **)vars[n].p==0)) error=2;
					break;
				case TYPE_DBL:
					if (*(double *)vars[n].p==0) error=1;
					break;
				case TYPE_STRLIST:
					if (((STRLIST *)vars[n].p)->count==0) error=3;
					break;
				case TYPE_BOOL:
					break;
			}
			if (error)
			{
				err3=err2;
				if (error==1)
					err3+="Must not be zero";
				if (error==2)
					err3+="Must not have zero length";
				if (error==3)
					err3+="Must not be empty";
				Errors.PushL(err3);
			}
		}
		// TODO: Add checks for options
	}
}


// Those are from configfile.cc
char    *ignore_SCRLF	(char *c);
char	*get_block	(char *c, LONGLINE& dest);
int 	load		(const char *fn, LONGLINE& dest);

/*
	Parse the config file
	Return: -1: Error opening file
		0: A line was bad
		1: OK
*/
int	SIMPLE_CONFIG::Parse()
{
	LONGLINE	buf;
	LONGLINE	name,value;
	int		n;
	char		*p;
	int		done;
	int		ok;

	n=load(fn.p,buf);
	if (n)
	{
		done=0;
		p=buf.p;
		while (!done)
		{
			name="";
			value="";
			ok=1;	// set this to 0 if the line has an error
			//pass spaces
			p=ignore_SCRLF(p);
			if (*p=='#')
			{
				// pass the comment and the EOL
				while ((*p) && (*p!='\n') && (*p!='\r'))
					p++;
				while ((*p=='\n') || (*p=='\r'))
					p++;
			} else
			{
				while ((*p) && (!isspace(*p)) && (*p!='='))
				{
					name+=*p;
					p++;
				}
				if ((*p!=' ') && (*p!='\t') && (*p!='='))
				{
					ok=0;
				} else
				{
					p=ignore_SCRLF(p);	// there are only spaces and tabs here
					if (*p!='=')		// there should be the =
					{
						ok=0;
					} else
					{
						p++;	// pass the =
						while ((*p==' ') || (*p=='\t'))	// pass spaces
							p++;

						p=get_block(p,value);	// get the value
						ConfAdd(name.p, value.p);	// set this
						CheckOpt(name.p, value.p);

						while ((*p) && (*p!='\n')  && (*p!='\r'))
							p++;
						while ((*p=='\n') || (*p=='\r'))	// pass EOL
							p++;
					}
				}
			}

			if (!ok)
				return(0);

			if (!(*p))
				done=1;
		}
	} else return(-1);
	
	return(1);
}

/*
	Read and parse the configuration file
*/
int	SIMPLE_CONFIG::Read(const char *fn, int flags)
{
	int		n;
	LONGLINE	err,*pll;
	int		ret=0;

	this->fn=fn;
	this->flags=flags;

	n=Parse();

	err="Configuration error in ";
	err+=fn;
	err+=": ";

	switch(n)
	{
		case -1:
			err+="Cannot open file: ";
			err+=strerror(errno);
			Errors.PushL(err);
			ret++;
			break;
		case 0:
			err+="File is bad or damaged.";
			Errors.PushL(err);
			ret++;
			break;
		case 1:
			CheckConfig();
			break;
	}

	if (Warnings.Count>0 && (flags & SCFL_HANDLE_WARNINGS))
	{
		Warnings.Rewind();
		while ((pll=Warnings.Get_Next())!=NULL)
			fprintf(stderr,"%s\n",pll->p);
	}

	if (Errors.Count>0 && (flags & SCFL_HANDLE_ERRORS))
	{
		Errors.Rewind();
		while ((pll=Errors.Get_Next())!=NULL)
			fprintf(stderr,"%s\n",pll->p);
	}

	if (Errors.Count>0 && (flags & SCFL_HALT_ERRORS))
	{
		fprintf(stderr,"Exiting due to configuration errors\n");
		exit(1);
	}

	if (Warnings.Count>0 && (flags & SCFL_HALT_WARNINGS))
	{
		fprintf(stderr,"Exiting due to configuration warnings\n");
		exit(2);
	}

	ret+=Warnings.Count+Errors.Count;
	
	return(ret);
}
