/****************************************/
/*					*/
/****************************************/
/* Start      : 14/05/2002		*/
/* Program ID : 			*/
/* Version    : 1.00			*/
/* End        : 			*/
/* Last Update: 14/05/2002		*/
/* Comments   : 			*/
/*                                      */
/****************************************/

#ifndef _VLIB_V_MISC_SIMPLECONFIGFILE_H
#define _VLIB_V_MISC_SIMPLECONFIGFILE_H

#include "v/lconfig.h"
#include "v/mem/longline.h"
#include "v/mem/dlist3.h"

struct CFG_VAR_OPT
{
	int	opt;
/*	int	value_int;
	int	value_double;*/
	void	*data;
	int	size;	// abstract size indicator
	u_32	flags;	// possible flags

	CFG_VAR_OPT	*next;
};

struct	CFG_VAR
{
	const char	*name;
	void		*p;
	const char	*dflt;
	u_32		type;
	u_32		flags;
	CFG_VAR_OPT	*opt;
};

#define	CFG_VAR_FL_CASE		0x0001	// Case sensitive
#define CFG_VAR_FL_EMPTY	0x0002	// Can be empty
#define CFG_VAR_FL_ZERO		0x0004	// Can be 0 - undefined for TYPE_FUNC
#define	CFG_VAR_FL_REHASH	0x0008	// This value can be reread/rehashed
#define	CFG_VAR_FL_NOINIT	0x0010	// Don't try to initialize this one (for use with TYPE_FUNC)

#define	CFG_VAR_TYPE_U32	1	// u_32
#define	CFG_VAR_TYPE_U16	2	// u_16
#define	CFG_VAR_TYPE_LL		3	// longline
#define	CFG_VAR_TYPE_CHAR_P	4	// char *. the string is strdup()'ed. Second call will free it and reallocate it
#define	CFG_VAR_TYPE_IP_LL	5	// IP address - stored as a LONGLINE
#define	CFG_VAR_TYPE_IP_CHAR_P	6	// IP address - stored as a char *
#define	CFG_VAR_TYPE_IP_U32	7	// IP address - stored as an u_32 in NBO
#define	CFG_VAR_TYPE_DBL	8	// double
#define	CFG_VAR_TYPE_FUNC	9	// call this function instead of storing the value somewhere
#define CFG_VAR_TYPE_STRLIST	10	// string list.. see v/mem/strlist.h
#define CFG_VAR_TYPE_BOOL	11	// Boolean.. Pointer points to a char (or something 1 byte)

#define	CFG_VAR_OPT_SET		1	// Restrict values

#define	CFG_VAR_OPT_SET_FL_CASE	0x0001	// Perform case sensitive match

/*
	hook function passed to CFG_VAR
	1st arg: the config entry that contains this function
	2nd arg: the string from the config file
	3rd arg: a pointer to a char *. If there is an error message, store it there. The buffer is freed when not needed
*/
typedef int (*cfg_hookfunc)(CFG_VAR *, const char *, char **);

#define CFG_HOOK_RET_OK		0
#define	CFG_HOOK_RET_WARNING	1
#define	CFG_HOOK_RET_ERROR	2

#define	SCFL_HANDLE_ERRORS	0x0001
#define SCFL_HANDLE_WARNINGS	0x0002
#define SCFL_HALT_ERRORS	0x0004
#define SCFL_HALT_WARNINGS	0x0008

#define SCFL_DEFAULT	(SCFL_HANDLE_ERRORS | SCFL_HANDLE_WARNINGS | SCFL_HALT_ERRORS | SCFL_HALT_WARNINGS)

class	SIMPLE_CONFIG
{
 private:
 public:
 	DL_List<LONGLINE>	Warnings;
	DL_List<LONGLINE>	Errors;
	CFG_VAR			*vars;
	LONGLINE		fn;
	int			flags;
 private:
 	void	ClearConfig	();
	void	CheckConfig	();
	void	ConfAdd		(const char *var, const char *value);
	void	CheckOpt	(const char *var, const char *value);
	int	Parse		();
 public:
	 	SIMPLE_CONFIG	(CFG_VAR *vars);
		~SIMPLE_CONFIG	();
	/*
		Return number of errors + warnings
	*/
	int	Read		(const char *fn, int flags=SCFL_DEFAULT);
};

/*
	Define this before including this file, to have
	sort names defined too
*/
//#ifdef SIMPLECONFIG_DEFINESIMPLE
/*
	Define this to prevent sort names from been
	defined.
 */
#ifndef SIMPLECONFIG_DONT_DEFINESIMPLE

#define	CASE		CFG_VAR_FL_CASE
#define EMPTY		CFG_VAR_FL_EMPTY
#define ZERO		CFG_VAR_FL_ZERO
#define	REHASH		CFG_VAR_FL_REHASH
#define NOINIT		CFG_VAR_FL_NOINIT

#define	TYPE_U32	CFG_VAR_TYPE_U32
#define	TYPE_U16	CFG_VAR_TYPE_U16
#define	TYPE_LL		CFG_VAR_TYPE_LL
#define	TYPE_CHAR_P	CFG_VAR_TYPE_CHAR_P
#define	TYPE_IP_LL	CFG_VAR_TYPE_IP_LL
#define	TYPE_IP_CHAR_P	CFG_VAR_TYPE_IP_CHAR_P
#define	TYPE_IP_U32	CFG_VAR_TYPE_IP_U32
#define	TYPE_DBL	CFG_VAR_TYPE_DBL
#define	TYPE_FUNC	CFG_VAR_TYPE_FUNC
#define TYPE_STRLIST	CFG_VAR_TYPE_STRLIST
#define	TYPE_BOOL	CFG_VAR_TYPE_BOOL

#define RET_OK		CFG_HOOK_RET_OK
#define	RET_WARN	CFG_HOOK_RET_WARNING
#define RET_WARNING	RET_WARN
#define	RET_ERROR	CFG_HOOK_RET_ERROR

#endif

/*
 Options:
	CFG_VAR_OPT_SET:
		
	data is a table of size elements. contains size possible values (strings)
	e.x.
		char		p[2]={"yes","no"};
		CFG_VAR_OPT	vopt={CFG_VAR_OPT_SET,p,2,NULL};
*/

#endif

