/****************************************/
/*  Time queue. (header)                */
/****************************************/
/* Start      : 15/9/98                 */
/* Program ID :                         */
/* Version    : 1.00                    */
/* End        :                         */
/* Last Update: 15/9/98                 */
/* Comments   : Store objects, that     */
/*   will be deleted afte expiration.   */
/****************************************/

/*
 Similiar to DL_List, but with timeout value. Keep the desired objects
 in the list, until timed-out.. then erase them. CheckExpire() should
 be called when possible, to check for expire objects and delete them.
 Objects have to be deletable (created with new)
 */

#ifndef _VLIB_V_MISC_TIMEQUEUE_H
#define _VLIB_V_MISC_TIMEQUEUE_H

#include "v/mem/dlist3.h"
#include "time.h"


template <class T> class TQOBJ
{
 private:
  void vvv(TQOBJ& p);
 public:
  TQOBJ();
  TQOBJ(TQOBJ& p);
  TQOBJ operator=(TQOBJ& p);
  T *d;
  time_t tstart;
  time_t tend;
  long ID;
};

template <class T> class TIMEQUEUE
{
 private:
  DL_List< TQOBJ<T> > d;
  long NextID;
  time_t cur;  // Current time... To keep different functions synchronized
  
 public:
 private:
  void Smit(T *d, time_t tstart, time_t tend);  // Submit (general)
  void Expire(long ID);  // Expire the object
  int Expired(TQOBJ<T> *p); // Check to see if the object has expired
 public:
  TIMEQUEUE();
  ~TIMEQUEUE();
  void Submit(T *d, time_t ttl);        // Submit with ttl (time to live)
  void SubmitUntil(T *d, time_t end);   // Submit with end_time
  void CheckExpire();                   // Expire the objects that should expire

  void Rewind();
  T *Get_Next();
  T *Get_Prev();
  void ClearList();
  time_t GetNextExpireTime();
  T *Get_NextExpired();
  /*Those will not change cur after calling SafeRewind()*/

  void SafeRewind();
  T *SafeGet_NextExpired();
  void SafeCheckExpire();
};

#endif
