/****************************************/
/*  SOCKET class (header)               */
/****************************************/
/* Start      : 11/10/98                */
/* Program ID :                         */
/* Version    : 2.00                    */
/* End        :                         */
/* Last Update:                         */
/* Comments   : 01/02/2002              */
/*                                      */
/****************************************/

#ifndef _VLIB_V_SOCKET_SOCKET3_H
#define _VLIB_V_SOCKET_SOCKET3_H

#include "sys/types.h"
#include "sys/socket.h"
#include "sys/un.h"
#include "netinet/in.h"

/*
 SRV:  socket->bind->listen->accept
 CLNT: socket->connect
 */

/*
	26/11/2001:	v1.10:
		Added unix domain sockets

	12/7/2001:	v1.00:

		SOCKET_* use the handle_error (default 1).
		handle_error == 0 : Don't do nothing. Return the return code of the called function
		handle_error == 1 : Display some info and exit
*/

/*
 SOCKET class

 */

#define _SOCKET_ERROR_SOCKET		1
#define _SOCKET_ERROR_BIND		2
#define _SOCKET_ERROR_LISTEN		3
#define _SOCKET_ERROR_ACCEPT		4
#define _SOCKET_ERROR_CONNECT		5
#define _SOCKET_ERROR_READ		6
#define _SOCKET_ERROR_WRITE		7
#define _SOCKET_ERROR_CLOSE		8
#define _SOCKET_ERROR_SETSOCKOPT	9
#define _SOCKET_ERROR_GETSOCKOPT	10
#define _SOCKET_ERROR_SOCKFD		11
#define _SOCKET_ERROR_SHUTDOWN		12

#define _SOCKET_MAX_QUEUE  1  //"very" max is 5


#define SOCK_SOCKET  1
#define SOCK_BIND    2
#define SOCK_LISTEN  4
#define SOCK_ACCEPT  8
#define SOCK_CONNECT 16

class SOCKET
{
 public:
//	long	status;
	int	sockfd;
	int	domain, type, protocol;
	union
	{
		sockaddr_in	sin;	/* the port is ntohs(sin.sin_port) */
		sockaddr_un	ssun;
	} addr, remote;
	int	last_error;		// The last error

 private:
	void	vvv		(const SOCKET& s);
 public:
 		SOCKET		();
	virtual	~SOCKET		();
		SOCKET		(const SOCKET &s);
	SOCKET&	operator=	(const SOCKET &s);
	void	Show();
};

class SOCKET_COMMON : public SOCKET
{
 public:
	int	handle_errors;	// Handle errors. Display some info and then exit..
	int	socket_auto;	// auto - socket, listen
 	int	ready_to_accept;	// This is set, if listen() succeeded

 private:
 	void		init		();
	void		vvv		(const SOCKET_COMMON& s);
 protected:
	int		Socket		(int domain, int type, int protocol);
						// Store the values and call socket()
 public:
			SOCKET_COMMON	();
	virtual		~SOCKET_COMMON	();
			SOCKET_COMMON	(const SOCKET_COMMON &s);
			SOCKET_COMMON	(const SOCKET &s);
	SOCKET_COMMON&	operator=	(const SOCKET_COMMON &s);
	SOCKET_COMMON&	operator=	(const SOCKET &s);

	ssize_t		Read		(char *buf,int buflen);
	ssize_t		Write		(char *buf,int buflen);

	void		ERROR		(int n);
	int		Close		();	// Perform a close() on the socket
	int		Shutdown	();	// Perform a shutdown() and a close() on the socket
	void		SetNonBlock	();
	void		SetBlock	();
	void 		SetLinger	(int onoff, int timeout);
	int 		HasError	();	// Return: 0: No error, 1: Socket has error

	int		Listen		(int backlog=_SOCKET_MAX_QUEUE);
	SOCKET		Accept		();	// Return a new SOCKET. SOCKET::sockfd==-1 on error
};

/*
	Abstract class.
	To be used by SOCKET_TCP and SOCKET_UDP
*/
class SOCKET_COMMON_TCPUDP : public SOCKET_COMMON
{
 private:
 public:
 			SOCKET_COMMON_TCPUDP	();
	virtual		~SOCKET_COMMON_TCPUDP	();

	int		Bind		(int address, short int port); // NBO both
	int		Bind		(short int port); // HBO <<< DIFERENT FROM ABOVE
	virtual int	Socket		()=0;	// child classes will implement this
	int		Connect		(int address, short int port);  // all in NBO
};

class SOCKET_TCP : public SOCKET_COMMON_TCPUDP
{
 public:
 			SOCKET_TCP	();
	virtual		~SOCKET_TCP	();

	int		Socket		();	// Return the result of socket()
};

class SOCKET_UDP : public SOCKET_COMMON_TCPUDP
{
 public:
 			SOCKET_UDP	();
	virtual		~SOCKET_UDP	();

	int		Socket		();	// Return the result of socket()
};

/*
	UNIX - STREAM
*/
class SOCKET_UNIX : public SOCKET_COMMON
{
 public:
 			SOCKET_UNIX	();
	virtual		~SOCKET_UNIX	();

	int		Bind		(const char *path);
	int		Socket		();
	int		Connect		(const char *path);
};

#endif


/*
 struct sockaddr_in {
  short int             sin_family;     // Address family
  unsigned short int    sin_port;       // Port number
  struct in_addr        sin_addr;       // Internet address

  // Pad to size of `struct sockaddr'.
  unsigned char         __pad[__SOCK_SIZE__ - sizeof(short int) -
                        sizeof(unsigned short int) - sizeof(struct in_addr)];
  };

  struct in_addr {
          __u32   s_addr;
  };

*/
