/****************************************/
/*  Parse a line                        */
/****************************************/
/* Start      : 19/9/98                 */
/* Program ID :                         */
/* Version    : 1.02                    */
/* End        :                         */
/* Last Update: 24/8/99                 */
/* Comments   : Make line parsing       */
/*              easier.                 */
/****************************************/

#include "parse.h"
#include "v/mem/dlist3.cc"

/******************* SEPERATOR ************************/

SEPERATOR::SEPERATOR()
{
 csepn=0;
 ssepn=0;
}

SEPERATOR::~SEPERATOR()
{
}

SEPERATOR::SEPERATOR(const SEPERATOR& p)
{
 vvv(p);
}

SEPERATOR SEPERATOR::operator=(const SEPERATOR& p)
{
 vvv(p);
 return(*this);
}

void SEPERATOR::vvv(const SEPERATOR& p)
{
 memcpy(csep,p.csep,_MAX_SEPERATORS);
 memcpy(ssep,p.ssep,_MAX_SEPERATORS*sizeof(char *));
}

int SEPERATOR::is_csep(const char *st)
{
 return(is_csep(*st));
}

int SEPERATOR::is_csep(char c)
{
 int n,r=0;

 for (n=0;n<csepn;n++)
  if (c==csep[n]) r=1;

 return(r);
}

int SEPERATOR::is_ssep(const char *st)
{
 int n,r=0,m;

 for (n=0;n<ssepn;n++)
 {
  if (!strncmp(st,ssep[n],(m=strlen(ssep[n]))))
  {
   r=m;
  }
 }
 return(r);
}

int SEPERATOR::AddSep(char c)
{
 int r;
 
 if ((r=find_sep(c))==-1)
  if (csepn<_MAX_SEPERATORS)
  {
   r=csepn;
   csep[csepn++]=c;
  }
 return(r);
}

int SEPERATOR::AddSep(const char *st)
{
 int r;

 if ((r=find_sep(st))==-1)
  if (ssepn<_MAX_SEPERATORS)
  {
   r=ssepn;
   ssep[ssepn++]=st;
  }

 return(r);
}

int SEPERATOR::DelSep(char c)
{
 int r;

 if ((r=find_sep(c))!=-1)
  csep[r]=csep[--csepn];

 return(r);
}

int SEPERATOR::DelSep(const char *st)
{
 int r;

 if ((r=find_sep(st))!=-1)
  ssep[r]=ssep[--ssepn];
 
 return(r);
}

int SEPERATOR::find_sep(char c)
{
 int n,r=-1;

 for (n=0;((n<csepn) && (r==-1));n++)
  if (csep[n]==c) r=n;

 return(r);
}

int SEPERATOR::find_sep(const char *st)
{
 int n,r=-1;
 
 for (n=0;((n<ssepn) && (r==-1));n++)
  if (!strcmp(ssep[n],st)) r=n;

 return(r);
}

int SEPERATOR::is_sep(const char *st)
{
 int r=0;

 if ((is_csep(*st)) || (is_ssep(st)))
  r=1;

 return(r);
}

int SEPERATOR::is_sep(const char *st, int *n)
{
 int r=0;
 int k;

 if ((k=is_csep(*st))) {r=1; *n=1;}
 else
  if ((k=is_ssep(st))) {r=1; *n=k;}

 return(r);
}

/******************************************************/

/********************** PARSE *************************/

PARSE::PARSE()
{
 flags=0;
}

PARSE::~PARSE()
{
}

PARSE::PARSE(PARSE &p)
{
 vvv(p);
}

PARSE PARSE::operator=(PARSE& p)
{
 vvv(p);
 return(*this);
}

void PARSE::vvv(PARSE& p)
{
 sep=p.sep;
 special=p.special;
 d=p.d;
 str=p.str;
 places=p.places;
}

int PARSE::AddSep(char c) { return(sep.AddSep(c)); }
int PARSE::AddSep(const char *st) { return(sep.AddSep(st)); }
int PARSE::DelSep(char c) { return(sep.DelSep(c)); }
int PARSE::DelSep(const char *st) { return(sep.AddSep(st)); }

int PARSE::AddSSep(char c) { return(special.AddSep(c)); }
int PARSE::AddSSep(const char *st) { return(special.AddSep(st)); }
int PARSE::DelSSep(char c) { return(special.DelSep(c)); }
int PARSE::DelSSep(const char *st) { return(special.AddSep(st)); }


void PARSE::Set(const char *st)
{
 str=st;
 Parse();
}

void PARSE::Set(const LONGLINE& st)
{
 str=st;
 Parse();
}

void PARSE::Parse()
{
 LONGLINE tmp;
 int n,m;
 int go_ahead;  /* 1 if a special seperator was found */
 char *p;
 size_t l;
 long last_pos;
 
 d.ClearList();
 places.Clear();
 p=str.p;
 go_ahead=0;
 last_pos=0;


 while (*p)
 {
  tmp="";
  n=m=l=0;
  last_pos=p-str.p;

  if (!go_ahead)
  {
   while ((!sep.is_sep((p+l),&n)) && (!special.is_sep((p+l),&m)) && (p[l]))
   {
    l++;
   }

   strncat(tmp,p,l);

   if (special.is_sep((p+l),&m))
    go_ahead=1;
  p+=m+n+l;

  } else
  {
   while (*p)
   {
    tmp+=*(p++);
   }
  }

//  if (strlen(tmp))  // WHY doy we need this ?
//  {
/* if ((!(flags|PARSEFLAG_IGNOREEMPTY)) || (strlen(tmp))) */
/** CORRECT?????????? **/
  if ((!(flags&PARSEFLAG_IGNOREEMPTY)) || (strlen(tmp)))
  {
   d.PushL(tmp);
   places.Add(last_pos);
  }
//  }
 }
}

char *PARSE::operator[](int n)
{
 char *c;
 LONGLINE *p;

 c=NULL;
 p=d[n];
 if (p) c=p->p;
 
 return(c);
}

char *PARSE::rest(int n)
{
 int m;
 char *c;

 m=places[n];
 //if (m==-1) c=NULL; else
 if (n<places.cmdN)
 {
 	m=places[n];
	c=str.p+(m);
 } else
 c=NULL;

 return(c);
}

/******************************************************/



