/****************************************/
/*  Parse a line (header)               */
/****************************************/
/* Start      : 19/9/98                 */
/* Program ID :                         */
/* Version    : 1.02                    */
/* End        :                         */
/* Last Update: 24/8/99                 */
/* Comments   : Make line parsing       */
/*              easier.                 */
/****************************************/

/*
 24/8/99 : v1.02: Added rest(). Returns the rest of the string after a block
 25/7/99 : v1.01: Added copy constructor and operator= to SEPERATOR and PARSE classes
*/

#ifndef _VLIB_V_STRING_PARSE_H
#define _VLIB_V_STRING_PARSE_H

#include "string.h"
#include "v/mem/longline.h"
#include "v/mem/dlist3.h"
#include "v/mem/morelists.h"

#define _MAX_SEPERATORS 10   /* up to 10 seperators can be specsified */

#define PARSEFLAG_IGNOREEMPTY 1 /* Ignore empty arguments */

class SEPERATOR   /* Class used from parser */
{
 private:
  char csep[_MAX_SEPERATORS];   /* character seperators */
  const char *ssep[_MAX_SEPERATORS]; /* string seperators    */

 public:
  int csepn;  /* Current characher seperators */
  int ssepn; /* Current string seperators    */

 private:
  void vvv(const SEPERATOR& p);
  
 public:
  SEPERATOR();
  ~SEPERATOR();
  SEPERATOR(const SEPERATOR& p);
  SEPERATOR operator=(const SEPERATOR& p);
  
  int is_csep(const char *st);
  int is_csep(char c);
  int is_ssep(const char *st);
  int find_sep(char c); /* Find a seperator. Return the index or -1 */
  int find_sep(const char *st);
  int is_sep(const char *st);
  int is_sep(const char *st, int *n);  /* n==the size of the seperator */

  int AddSep(char c);   /* Add another seperator. */
  int AddSep(const char *st); /* Return -1 if error     */
  int DelSep(char c);   /* Remove a seperator.    */
  int DelSep(const char *st); /* Return -1 if error     */

};

class PARSE
{
 private:
  SEPERATOR sep;      /* Common seperators */
  SEPERATOR special;  /* Special seperators ( After one of them is found, everything else is a block ) */
  DL_List<LONGLINE> d;
  LONGLINE str;
  NUMLIST places;    /* Store the position of the beggining of each block */
  
 public:
  int flags;

 private:
  void Parse();
  void vvv(PARSE& p);
  
 public:
  PARSE();
  ~PARSE();
  PARSE(PARSE &p);
  PARSE operator=(PARSE& p);

  int AddSep(char c);     /*  Add a common seperator  */
  int AddSep(const char *st);   /*  Add a common seperator  */
  int DelSep(char c);     /*  Add a common seperator  */
  int DelSep(const char *st);   /*  Add a common seperator  */
  
  int AddSSep(char c);    /*  Add a special seperator  */
  int AddSSep(const char *st);  /*  Add a special seperator  */
  int DelSSep(char c);    /*  Add a special seperator  */
  int DelSSep(const char *st);  /*  Add a special seperator  */

  void Set(const char *st);     /* Set the string. !!WARNING!! It copies the */
                          /* string to a LONGSTRING!!!                 */
  void Set(const LONGLINE& st); /* ...as above...*/
  char *operator[](int n); /* 1st = 0 */
  char *rest(int n); /* Return the rest of the string after n block (n,n+1,n+2...) */
};

#endif

